@@ -20,6 +20,7 @@
 package accounts
 
 import (
+	"encoding/base64"
 	"errors"
 	"io/ioutil"
 	"os"
@@ -30,19 +31,23 @@ import (
 	"sync"
 
 	dbus "github.com/godbus/dbus"
+	"github.com/linuxdeepin/dde-daemon/accounts/users"
 	authenticate "github.com/linuxdeepin/go-dbus-factory/com.deepin.daemon.authenticate"
-	"pkg.deepin.io/dde/daemon/accounts/users"
-	glib "pkg.deepin.io/gir/glib-2.0"
-	"pkg.deepin.io/lib/dbusutil"
-	"pkg.deepin.io/lib/gdkpixbuf"
-	"pkg.deepin.io/lib/strv"
-	dutils "pkg.deepin.io/lib/utils"
+	glib "github.com/linuxdeepin/go-gir/glib-2.0"
+	"github.com/linuxdeepin/go-lib/dbusutil"
+	"github.com/linuxdeepin/go-lib/gdkpixbuf"
+	"github.com/linuxdeepin/go-lib/procfs"
+	"github.com/linuxdeepin/go-lib/strv"
+	dutils "github.com/linuxdeepin/go-lib/utils"
 )
 
 const (
 	defaultUserIcon          = "file:///var/lib/AccountsService/icons/default.png"
 	defaultUserBackgroundDir = "/usr/share/wallpapers/deepin/"
 
+	controlCenterPath = "/usr/bin/dde-control-center"
+	deepinDaemonDir   = "/usr/lib/deepin-daemon/"
+
 	maxWidth  = 200
 	maxHeight = 200
 )
@@ -67,6 +72,7 @@ const (
 	confKeyShortTimeFormat    = "ShortTimeFormat"
 	confKeyLongTimeFormat     = "LongTimeFormat"
 	confKeyWeekBegins         = "WeekBegins"
+	confKeyPasswordHint       = "PasswordHint"
 
 	defaultUse24HourFormat = true
 	defaultWeekdayFormat   = 0
@@ -101,6 +107,7 @@ type User struct {
 	Locale          string
 	Layout          string
 	IconFile        string
+	PasswordHint    string
 	Use24HourFormat bool
 	WeekdayFormat   int32
 	ShortDateFormat int32
@@ -175,180 +182,45 @@ func NewUser(userPath string, service *dbusutil.Service, ignoreErr bool) (*User,
 		PasswordLastChange: int32(shadowInfo.LastChange),
 	}
 
-	u.AccountType = u.getAccountType()
-	u.IconList = u.getAllIcons()
-	u.Groups = u.getGroups()
-
-	// NOTICE(jouyouyun): Got created time,  not accurate, can only be used as a reference
-	u.CreatedTime, err = u.getCreatedTime()
-	if err != nil {
-		logger.Warning("Failed to get created time:", err)
-	}
-
 	updateConfigPath(userInfo.Name)
+	u.AccountType = u.getAccountType()
+	u.Groups = u.getGroups()
+	loadUserConfigInfo(u)
 
-	kf, err := dutils.NewKeyFileFromFile(
-		path.Join(userConfigDir, userInfo.Name))
-	if err != nil {
-		xSession, _ := users.GetDefaultXSession()
-		u.XSession = xSession
-		u.SystemAccount = false
-		u.Layout = getDefaultLayout()
-		u.setLocale(getDefaultLocale())
-		u.IconFile = defaultUserIcon
-		defaultUserBackground := getDefaultUserBackground()
-		u.DesktopBackgrounds = []string{defaultUserBackground}
-		u.GreeterBackground = defaultUserBackground
-		u.Use24HourFormat = defaultUse24HourFormat
-		u.UUID = dutils.GenUuid()
-		u.WeekdayFormat = defaultWeekdayFormat
-		u.ShortDateFormat = defaultShortDateFormat
-		u.LongDateFormat = defaultLongDateFormat
-		u.ShortTimeFormat = defaultShortTimeFormat
-		u.LongTimeFormat = defaultLongTimeFormat
-		u.WeekBegins = defaultWeekBegins
-		u.Workspace = defaultWorkspace
-
-		err = u.writeUserConfig()
-		if err != nil {
-			logger.Warning(err)
-		}
-		return u, nil
-	}
-	defer kf.Free()
-
-	var isSave = false
-	xSession, _ := kf.GetString(confGroupUser, confKeyXSession)
-	u.XSession = xSession
-	if u.XSession == "" {
-		xSession, _ = users.GetDefaultXSession()
-		u.XSession = xSession
-		isSave = true
-	}
-	_, err = kf.GetBoolean(confGroupUser, confKeySystemAccount)
-	// only show non system account
-	u.SystemAccount = false
-	if err != nil {
-		isSave = true
-	}
-	locale, _ := kf.GetString(confGroupUser, confKeyLocale)
-	u.setLocale(locale)
-	if locale == "" {
-		u.setLocale(getDefaultLocale())
-		isSave = true
-	}
-	layout, _ := kf.GetString(confGroupUser, confKeyLayout)
-	u.Layout = layout
-	if layout == "" {
-		u.Layout = getDefaultLayout()
-		isSave = true
-	}
-	icon, _ := kf.GetString(confGroupUser, confKeyIcon)
-	u.IconFile = icon
-	if u.IconFile == "" {
-		u.IconFile = defaultUserIcon
-		isSave = true
-	}
-
-	u.customIcon, _ = kf.GetString(confGroupUser, confKeyCustomIcon)
-
-	// CustomIcon is the newly added field in the configuration file
-	if u.customIcon == "" {
-		if u.IconFile != defaultUserIcon && !isStrInArray(u.IconFile, u.IconList) {
-			// u.IconFile is a custom icon, not a standard icon
-			u.customIcon = u.IconFile
-			isSave = true
-		}
-	}
-
-	u.IconList = u.getAllIcons()
-
-	_, desktopBgs, _ := kf.GetStringList(confGroupUser, confKeyDesktopBackgrounds)
-	u.DesktopBackgrounds = desktopBgs
-	if len(desktopBgs) == 0 {
-		u.DesktopBackgrounds = []string{getDefaultUserBackground()}
-		isSave = true
-	}
-
-	greeterBg, ok := getUserGreeterBackground(kf)
-	if ok {
-		u.GreeterBackground = greeterBg
-	} else {
-		u.GreeterBackground = getDefaultUserBackground()
-		isSave = true
-	}
-
-	_, u.HistoryLayout, _ = kf.GetStringList(confGroupUser, confKeyHistoryLayout)
-	if !strv.Strv(u.HistoryLayout).Contains(u.Layout) {
-		u.HistoryLayout = append(u.HistoryLayout, u.Layout)
-		isSave = true
-	}
-
-	u.Use24HourFormat, err = kf.GetBoolean(confGroupUser, confKeyUse24HourFormat)
-	if err != nil {
-		u.Use24HourFormat = defaultUse24HourFormat
-		isSave = true
-	}
-
-	u.WeekdayFormat, err = kf.GetInteger(confGroupUser, confKeyWeekdayFormat)
-	if err != nil {
-		u.WeekdayFormat = defaultWeekdayFormat
-		isSave = true
-	}
-
-	u.ShortDateFormat, err = kf.GetInteger(confGroupUser, confKeyShortDateFormat)
-	if err != nil {
-		u.ShortDateFormat = defaultShortDateFormat
-		isSave = true
-	}
-
-	u.LongDateFormat, err = kf.GetInteger(confGroupUser, confKeyLongDateFormat)
-	if err != nil {
-		u.LongDateFormat = defaultLongDateFormat
-		isSave = true
-	}
-
-	u.ShortTimeFormat, err = kf.GetInteger(confGroupUser, confKeyShortTimeFormat)
-	if err != nil {
-		u.ShortTimeFormat = defaultShortTimeFormat
-		isSave = true
-	}
-
-	u.LongTimeFormat, err = kf.GetInteger(confGroupUser, confKeyLongTimeFormat)
-	if err != nil {
-		u.LongTimeFormat = defaultLongTimeFormat
-		isSave = true
-	}
-
-	u.WeekBegins, err = kf.GetInteger(confGroupUser, confKeyWeekBegins)
-	if err != nil {
-		u.WeekBegins = defaultWeekBegins
-		isSave = true
-	}
-
-	u.UUID, err = kf.GetString(confGroupUser, confKeyUUID)
-	if err != nil || u.UUID == "" {
-		u.UUID = dutils.GenUuid()
-		isSave = true
-	}
-
-	u.Workspace, err = kf.GetInteger(confGroupUser, confKeyWorkspace)
-	if err != nil || u.Workspace == 0 {
-		u.Workspace = defaultWorkspace
-		isSave = true
-	}
-
-	if isSave {
-		err := u.writeUserConfig()
-		if err != nil {
-			logger.Warning(err)
-		}
-	}
-
-	u.checkLeftSpace()
 	return u, nil
 }
 
+func NewUdcpUser(usrId uint32, service *dbusutil.Service, groups []string) (*User, error) {
+	var err error
+	if users.ExistPwUid(usrId) != 0 {
+		return nil, errors.New("no such user id")
+	}
+
+	userName := users.GetPwName(usrId)
+
+	var u = &User{
+		service:            service,
+		UserName:           users.GetPwName(usrId),
+		FullName:           users.GetPwGecos(usrId),
+		Uid:                users.GetPwUid(usrId),
+		Gid:                users.GetPwGid(usrId),
+		HomeDir:            users.GetPwDir(usrId),
+		Shell:              users.GetPwShell(usrId),
+		AutomaticLogin:     users.IsAutoLoginUser(userName),
+		NoPasswdLogin:      users.CanNoPasswdLogin(userName),
+		Locked:             false,
+		PasswordStatus:     users.PasswordStatusUsable,
+		MaxPasswordAge:     30,
+		PasswordLastChange: 18737,
+	}
+
+	u.AccountType = users.UserTypeUdcp
+	u.Groups = groups
+
+	loadUserConfigInfo(u)
+	return u, err
+}
+
 func getUserGreeterBackground(kf *glib.KeyFile) (string, bool) {
 	greeterBg, _ := kf.GetString(confGroupUser, confKeyGreeterBackground)
 	if greeterBg == "" {
@@ -376,7 +248,7 @@ func (u *User) getAllIcons() []string {
 }
 
 func (u *User) getGroups() []string {
-	groups, err := users.GetUserGroups(u.UserName)
+	groups, err := users.GetUserGroups(u.UserName, u.Gid)
 	if err != nil {
 		logger.Warning("failed to get user groups:", err)
 		return nil
@@ -511,6 +383,11 @@ func (u *User) updatePropGroups() {
 	u.PropsMu.Unlock()
 }
 
+func (u *User) updatePropGroupsNoLock() {
+	newVal := u.getGroups()
+	u.setPropGroups(newVal)
+}
+
 func (u *User) updatePropAutomaticLogin() {
 	newVal := users.IsAutoLoginUser(u.UserName)
 	u.PropsMu.Lock()
@@ -523,7 +400,11 @@ func (u *User) updatePropsPasswd(uInfo *users.UserInfo) {
 	var oldUserName string
 
 	u.PropsMu.Lock()
-	u.setPropGid(uInfo.Gid)
+	if u.Gid != uInfo.Gid {
+		// gid 被修改
+		u.setPropGid(uInfo.Gid)
+		u.updatePropGroupsNoLock()
+	}
 
 	if u.UserName != uInfo.Name {
 		oldUserName = u.UserName
@@ -565,6 +446,54 @@ func (u *User) getAccountType() int32 {
 	return users.UserTypeStandard
 }
 
+func (u *User) checkIsControlCenter(sender dbus.Sender) bool {
+	pid, err := u.service.GetConnPID(string(sender))
+	if err != nil {
+		logger.Warning(err)
+		return false
+	}
+
+	p := procfs.Process(pid)
+	exe, err := p.Exe()
+	if err != nil {
+		logger.Warning(err)
+		return false
+	}
+
+	if exe == controlCenterPath {
+		return true
+	}
+
+	return false
+}
+
+func (u *User) checkIsDeepinDaemon(sender dbus.Sender) bool {
+	pid, err := u.service.GetConnPID(string(sender))
+	if err != nil {
+		logger.Warning(err)
+		return false
+	}
+
+	p := procfs.Process(pid)
+	exe, err := p.Exe()
+	if err != nil {
+		logger.Warning(err)
+		return false
+	}
+
+	abs, err := filepath.Abs(exe)
+	if err != nil {
+		logger.Warning(err)
+		return false
+	}
+
+	if strings.HasPrefix(abs, deepinDaemonDir) {
+		return true
+	}
+
+	return false
+}
+
 func (u *User) checkAuth(sender dbus.Sender, selfPass bool, actionId string) error {
 	uid, err := u.service.GetConnUID(string(sender))
 	if err != nil {
@@ -629,9 +558,13 @@ func (u *User) clearData() {
 		}
 	}
 
-	u.clearFingers()
 }
 
+// 删除生物认证信息,TODO (人脸，虹膜)
+func (u *User) clearBiometricChara() {
+	u.clearFingers()
+
+}
 func (u *User) clearFingers() {
 	logger.Debug("clearFingers")
 
@@ -752,3 +685,190 @@ func updateConfigPath(username string) {
 		logger.Warning("Failed to update config:", username)
 	}
 }
+
+// 从配置文件中加载用户的配置信息
+func loadUserConfigInfo(u *User) {
+	var err error
+
+	u.IconList = u.getAllIcons()
+
+	// NOTICE(jouyouyun): Got created time,  not accurate, can only be used as a reference
+	u.CreatedTime, err = u.getCreatedTime()
+	if err != nil {
+		logger.Warning("Failed to get created time:", err)
+	}
+
+	kf, err := dutils.NewKeyFileFromFile(
+		path.Join(userConfigDir, u.UserName))
+	if err != nil {
+		xSession, _ := users.GetDefaultXSession()
+		u.XSession = xSession
+		u.SystemAccount = false
+		u.Layout = getDefaultLayout()
+		u.setLocale(getDefaultLocale())
+		u.IconFile = defaultUserIcon
+		defaultUserBackground := getDefaultUserBackground()
+		u.DesktopBackgrounds = []string{defaultUserBackground}
+		u.GreeterBackground = defaultUserBackground
+		u.Use24HourFormat = defaultUse24HourFormat
+		u.UUID = dutils.GenUuid()
+		u.WeekdayFormat = defaultWeekdayFormat
+		u.ShortDateFormat = defaultShortDateFormat
+		u.LongDateFormat = defaultLongDateFormat
+		u.ShortTimeFormat = defaultShortTimeFormat
+		u.LongTimeFormat = defaultLongTimeFormat
+		u.WeekBegins = defaultWeekBegins
+		u.Workspace = defaultWorkspace
+
+		err = u.writeUserConfig()
+		if err != nil {
+			logger.Warning(err)
+		}
+
+		return
+	}
+	defer kf.Free()
+
+	var isSave = false
+	xSession, _ := kf.GetString(confGroupUser, confKeyXSession)
+	u.XSession = xSession
+	if u.XSession == "" {
+		xSession, _ = users.GetDefaultXSession()
+		u.XSession = xSession
+		isSave = true
+	}
+	_, err = kf.GetBoolean(confGroupUser, confKeySystemAccount)
+	// only show non system account
+	u.SystemAccount = false
+	if err != nil {
+		isSave = true
+	}
+	locale, _ := kf.GetString(confGroupUser, confKeyLocale)
+	u.setLocale(locale)
+	if locale == "" {
+		u.setLocale(getDefaultLocale())
+		isSave = true
+	}
+	layout, _ := kf.GetString(confGroupUser, confKeyLayout)
+	u.Layout = layout
+	if layout == "" {
+		u.Layout = getDefaultLayout()
+		isSave = true
+	}
+	icon, _ := kf.GetString(confGroupUser, confKeyIcon)
+	u.IconFile = icon
+	if u.IconFile == "" {
+		u.IconFile = defaultUserIcon
+		isSave = true
+	}
+
+	u.customIcon, _ = kf.GetString(confGroupUser, confKeyCustomIcon)
+
+	// CustomIcon is the newly added field in the configuration file
+	if u.customIcon == "" {
+		if u.IconFile != defaultUserIcon && !isStrInArray(u.IconFile, u.IconList) {
+			// u.IconFile is a custom icon, not a standard icon
+			u.customIcon = u.IconFile
+			isSave = true
+		}
+	}
+
+	u.IconList = u.getAllIcons()
+
+	_, desktopBgs, _ := kf.GetStringList(confGroupUser, confKeyDesktopBackgrounds)
+	u.DesktopBackgrounds = desktopBgs
+	if len(desktopBgs) == 0 {
+		u.DesktopBackgrounds = []string{getDefaultUserBackground()}
+		isSave = true
+	}
+
+	greeterBg, ok := getUserGreeterBackground(kf)
+	if ok {
+		u.GreeterBackground = greeterBg
+	} else {
+		u.GreeterBackground = getDefaultUserBackground()
+		isSave = true
+	}
+
+	_, u.HistoryLayout, _ = kf.GetStringList(confGroupUser, confKeyHistoryLayout)
+	if !strv.Strv(u.HistoryLayout).Contains(u.Layout) {
+		u.HistoryLayout = append(u.HistoryLayout, u.Layout)
+		isSave = true
+	}
+
+	u.Use24HourFormat, err = kf.GetBoolean(confGroupUser, confKeyUse24HourFormat)
+	if err != nil {
+		u.Use24HourFormat = defaultUse24HourFormat
+		isSave = true
+	}
+
+	u.WeekdayFormat, err = kf.GetInteger(confGroupUser, confKeyWeekdayFormat)
+	if err != nil {
+		u.WeekdayFormat = defaultWeekdayFormat
+		isSave = true
+	}
+
+	u.ShortDateFormat, err = kf.GetInteger(confGroupUser, confKeyShortDateFormat)
+	if err != nil {
+		u.ShortDateFormat = defaultShortDateFormat
+		isSave = true
+	}
+
+	u.LongDateFormat, err = kf.GetInteger(confGroupUser, confKeyLongDateFormat)
+	if err != nil {
+		u.LongDateFormat = defaultLongDateFormat
+		isSave = true
+	}
+
+	u.ShortTimeFormat, err = kf.GetInteger(confGroupUser, confKeyShortTimeFormat)
+	if err != nil {
+		u.ShortTimeFormat = defaultShortTimeFormat
+		isSave = true
+	}
+
+	u.LongTimeFormat, err = kf.GetInteger(confGroupUser, confKeyLongTimeFormat)
+	if err != nil {
+		u.LongTimeFormat = defaultLongTimeFormat
+		isSave = true
+	}
+
+	u.WeekBegins, err = kf.GetInteger(confGroupUser, confKeyWeekBegins)
+	if err != nil {
+		u.WeekBegins = defaultWeekBegins
+		isSave = true
+	}
+
+	u.UUID, err = kf.GetString(confGroupUser, confKeyUUID)
+	if err != nil || u.UUID == "" {
+		u.UUID = dutils.GenUuid()
+		isSave = true
+	}
+
+	u.Workspace, err = kf.GetInteger(confGroupUser, confKeyWorkspace)
+	if err != nil || u.Workspace == 0 {
+		u.Workspace = defaultWorkspace
+		isSave = true
+	}
+
+	passwordHint, err := kf.GetString(confGroupUser, confKeyPasswordHint)
+	if err != nil {
+		u.PasswordHint = ""
+	} else {
+		decodeHint, err := base64.StdEncoding.DecodeString(passwordHint)
+		if err != nil {
+			logger.Warning(err)
+			u.PasswordHint = ""
+		} else {
+			u.PasswordHint = string(decodeHint)
+		}
+	}
+
+	if isSave {
+		err := u.writeUserConfig()
+		if err != nil {
+			logger.Warning(err)
+		}
+	}
+
+	u.checkLeftSpace()
+}
