@@ -19,7 +19,13 @@
 
 package accounts
 
+/*
+#include <shadow.h>
+typedef struct spwd cspwd;
+*/
+import "C"
 import (
+	"encoding/base64"
 	"errors"
 	"fmt"
 	"os"
@@ -30,13 +36,13 @@ import (
 	"time"
 
 	dbus "github.com/godbus/dbus"
-	"pkg.deepin.io/dde/api/lang_info"
-	"pkg.deepin.io/dde/daemon/accounts/users"
-	"pkg.deepin.io/lib/dbusutil"
-	"pkg.deepin.io/lib/gdkpixbuf"
-	"pkg.deepin.io/lib/imgutil"
-	"pkg.deepin.io/lib/strv"
-	dutils "pkg.deepin.io/lib/utils"
+	"github.com/linuxdeepin/dde-api/lang_info"
+	"github.com/linuxdeepin/dde-daemon/accounts/users"
+	"github.com/linuxdeepin/go-lib/dbusutil"
+	"github.com/linuxdeepin/go-lib/gdkpixbuf"
+	"github.com/linuxdeepin/go-lib/imgutil"
+	"github.com/linuxdeepin/go-lib/strv"
+	dutils "github.com/linuxdeepin/go-lib/utils"
 )
 
 const (
@@ -57,6 +63,11 @@ func (u *User) SetFullName(sender dbus.Sender, name string) *dbus.Error {
 		return dbusutil.ToError(err)
 	}
 
+	if !u.checkIsControlCenter(sender) {
+		logger.Debug("[SetLocale] access denied: only dde-control-center allowed")
+		return dbusutil.ToError(fmt.Errorf("only dde-control-center allowed to call this method"))
+	}
+
 	u.PropsMu.Lock()
 	defer u.PropsMu.Unlock()
 
@@ -142,6 +153,16 @@ func (u *User) SetShell(sender dbus.Sender, shell string) *dbus.Error {
 func (u *User) SetPassword(sender dbus.Sender, password string) *dbus.Error {
 	logger.Debug("[SetPassword] start ...")
 
+	// set password from UnionID
+	if password == "" {
+		err := u.setPwdWithUnionID(sender)
+		if err != nil {
+			return dbusutil.ToError(err)
+		} else {
+			return nil
+		}
+	}
+
 	err := u.checkAuth(sender, false, "")
 	if err != nil {
 		logger.Debug("[SetPassword] access denied:", err)
@@ -157,7 +178,7 @@ func (u *User) SetPassword(sender dbus.Sender, password string) *dbus.Error {
 		}
 		count--
 		if count == 0 {
-			return dbusutil.ToError(errors.New("shadow file error"))
+			return dbusutil.ToError(err)
 		}
 		time.Sleep(time.Second)
 	}
@@ -167,6 +188,11 @@ func (u *User) SetPassword(sender dbus.Sender, password string) *dbus.Error {
 		return dbusutil.ToError(err)
 	}
 
+	err = removeLoginKeyring(u)
+	if err != nil {
+		logger.Warningf("DoAction: remove login keyring failed: %v", err)
+	}
+
 	u.PropsMu.Lock()
 	defer u.PropsMu.Unlock()
 
@@ -365,6 +391,11 @@ func (u *User) SetLocale(sender dbus.Sender, locale string) *dbus.Error {
 		return dbusutil.ToError(err)
 	}
 
+	if !u.checkIsDeepinDaemon(sender) {
+		logger.Debug("[SetLocale] access denied: only deepin daemons allowed")
+		return dbusutil.ToError(fmt.Errorf("only deepin daemons allowed to call this method"))
+	}
+
 	if !lang_info.IsSupportedLocale(locale) {
 		err := fmt.Errorf("invalid locale %q", locale)
 		logger.Debug("[SetLocale]", err)
@@ -508,7 +539,17 @@ func (u *User) SetIconFile(sender dbus.Sender, iconURI string) *dbus.Error {
 func (u *User) DeleteIconFile(sender dbus.Sender, icon string) *dbus.Error {
 	logger.Debug("[DeleteIconFile] icon:", icon)
 
-	err := u.checkAuth(sender, true, "")
+	dir, err := filepath.Abs(filepath.Dir(icon))
+	if err != nil || dir != userCustomIconsDir {
+		return dbusutil.ToError(fmt.Errorf("%s is not in %s", icon, userCustomIconsDir))
+	}
+
+	base := filepath.Base(icon)
+	if !strings.HasPrefix(base, u.UserName) {
+		return dbusutil.ToError(fmt.Errorf("%s is not belong to %s", icon, u.UserName))
+	}
+
+	err = u.checkAuth(sender, true, "")
 	if err != nil {
 		logger.Debug("[DeleteIconFile] access denied:", err)
 		return dbusutil.ToError(err)
@@ -935,3 +976,58 @@ func (u *User) SetWeekBegins(sender dbus.Sender, value int32) *dbus.Error {
 	}
 	return nil
 }
+
+type ExpiredStatus int
+
+const (
+	expiredStatusNormal ExpiredStatus = iota
+	expiredStatusExpiredSoon
+	expiredStatusExpiredAlready
+)
+
+const secondsPerDay = 60 * 60 * 24
+
+func (u *User) PasswordExpiredInfo() (expiredStatus ExpiredStatus, dayLeft int64, busErr *dbus.Error) {
+	var pw *C.cspwd
+	pw = C.getspnam(C.CString(u.UserName))
+	if pw == nil {
+		return expiredStatusNormal, 0, dbusutil.ToError(fmt.Errorf("get passwd for %s failed", u.UserName))
+	}
+
+	var spMax = int64(pw.sp_max)
+	var spWarn = int64(pw.sp_warn)
+	var spLastChg = int64(pw.sp_lstchg)
+
+	if spLastChg == 0 {
+		// expired
+		return expiredStatusExpiredAlready, 0, nil
+	}
+	if spMax == -1 {
+		// never expired
+		return expiredStatusNormal, -1, nil
+	}
+
+	// pam_unix/passverify.c
+	curDays := time.Now().Unix() / secondsPerDay
+	daysLeft := spLastChg + spMax - curDays
+
+	if daysLeft < 0 {
+		return expiredStatusExpiredAlready, daysLeft, nil
+	} else if spWarn > daysLeft {
+		return expiredStatusExpiredSoon, daysLeft, nil
+	}
+	return expiredStatusNormal, daysLeft, nil
+}
+
+func (u *User) SetPasswordHint(hint string) (busErr *dbus.Error) {
+	encodeHint := base64.StdEncoding.EncodeToString([]byte(hint))
+	err := u.writeUserConfigWithChange(confKeyPasswordHint, encodeHint)
+	if err == nil {
+		u.setPropPasswordHint(hint)
+	}
+	return dbusutil.ToError(err)
+}
+
+func (u *User) GetReminderInfo() (info LoginReminderInfo, dbusErr *dbus.Error) {
+	return getLoginReminderInfo(u.UserName), nil
+}
