@@ -20,10 +20,11 @@
 package users
 
 /*
-#cgo CFLAGS: -Wall -g
+#cgo CFLAGS: -W -Wall -g  -fstack-protector-strong -D_FORTITY_SOURCE=1 -fPIC
 #cgo LDFLAGS: -lcrypt
 
 #include <stdlib.h>
+#include <shadow.h>
 #include "passwd.h"
 */
 import "C"
@@ -31,7 +32,10 @@ import "C"
 import (
 	"fmt"
 	"io/ioutil"
+	"math"
 	"os"
+	"os/user"
+	"strconv"
 	"strings"
 	"sync"
 	"unsafe"
@@ -48,6 +52,60 @@ func EncodePasswd(words string) string {
 	return C.GoString(C.mkpasswd(cwords))
 }
 
+func ExistPwUid(uid uint32) int {
+	return int(C.exist_pw_uid(C.uint(uid)))
+}
+
+func GetPwName(uid uint32) string {
+	return C.GoString(C.get_pw_name(C.uint(uid)))
+}
+
+func GetPwGecos(uid uint32) string {
+	var pwGecos string
+	pwGecos = C.GoString(C.get_pw_gecos(C.uint(uid)))
+
+	return strings.Split(pwGecos, ",")[0]
+}
+
+func GetPwUid(uid uint32) string {
+	return strconv.FormatUint(uint64(C.get_pw_uid(C.uint(uid))), 10)
+}
+
+func GetPwGid(uid uint32) string {
+	return strconv.FormatUint(uint64(C.get_pw_gid(C.uint(uid))), 10)
+}
+
+func GetPwDir(uid uint32) string {
+	return C.GoString(C.get_pw_dir(C.uint(uid)))
+}
+
+func GetPwShell(uid uint32) string {
+	return C.GoString(C.get_pw_shell(C.uint(uid)))
+}
+
+type originShadowInfo struct {
+	Name       string
+	LastChange int
+	MaxDays    int
+	ShadowPwdp string // password status
+}
+
+func getSpwd(username string) (*originShadowInfo, error) {
+	cname := C.CString(username)
+	defer C.free(unsafe.Pointer(cname))
+	spwd := C.getspnam(cname)
+	if spwd == nil {
+		return &originShadowInfo{}, fmt.Errorf("no such user name: %v", username)
+	}
+	sInfo := originShadowInfo{
+		Name:       C.GoString(spwd.sp_namp),
+		LastChange: int(spwd.sp_lstchg),
+		MaxDays:    int(spwd.sp_max),
+		ShadowPwdp: C.GoString(spwd.sp_pwdp),
+	}
+	return &sInfo, nil
+}
+
 // password: has been crypt
 func updatePasswd(password, username string) error {
 	status := C.lock_shadow_file()
@@ -102,7 +160,18 @@ func updatePasswd(password, username string) error {
 		return fmt.Errorf("The username not exist.")
 	}
 
-	return writeStrvToFile(datas, userFileShadow, 0600)
+	err = writeStrvToFile(datas, userFileShadow, math.MaxUint32)
+	if err != nil {
+		return nil
+	}
+
+	usr, _ := user.Lookup("root")
+	grp, _ := user.LookupGroup("shadow")
+	uid, _ := strconv.Atoi(usr.Uid)
+	gid, _ := strconv.Atoi(grp.Gid)
+	err = os.Chown(userFileShadow, uid, gid)
+
+	return err
 }
 
 func writeStrvToFile(datas []string, file string, mode os.FileMode) error {
@@ -135,8 +204,13 @@ func writeStrvToFile(datas []string, file string, mode os.FileMode) error {
 		return err
 	}
 
-	_ = os.Rename(file+".bak~", file)
-	_ = os.Chmod(file, mode)
+	info, err := os.Stat(file)
+	if err == nil && mode == math.MaxUint32 {
+		mode = info.Mode()
+	}
+
+	os.Rename(file+".bak~", file)
+	os.Chmod(file, mode)
 
 	return nil
 }
