@@ -20,6 +20,7 @@
 package shortcuts
 
 import (
+	"fmt"
 	"os"
 	"os/exec"
 	"strings"
@@ -29,21 +30,82 @@ import (
 	"github.com/godbus/dbus"
 	daemon "github.com/linuxdeepin/go-dbus-factory/com.deepin.daemon.daemon"
 	wm "github.com/linuxdeepin/go-dbus-factory/com.deepin.wm"
+	gio "github.com/linuxdeepin/go-gir/gio-2.0"
+	"github.com/linuxdeepin/go-lib/gettext"
+	"github.com/linuxdeepin/go-lib/keyfile"
+	"github.com/linuxdeepin/go-lib/log"
+	"github.com/linuxdeepin/go-lib/pinyin_search"
+	dutils "github.com/linuxdeepin/go-lib/utils"
 	x "github.com/linuxdeepin/go-x11-client"
 	"github.com/linuxdeepin/go-x11-client/ext/record"
 	"github.com/linuxdeepin/go-x11-client/util/keybind"
 	"github.com/linuxdeepin/go-x11-client/util/keysyms"
 	"github.com/linuxdeepin/go-x11-client/util/wm/ewmh"
-	"pkg.deepin.io/dde/daemon/keybinding/util"
-	gio "pkg.deepin.io/gir/gio-2.0"
-	"pkg.deepin.io/lib/gettext"
-	"pkg.deepin.io/lib/log"
-	"pkg.deepin.io/lib/pinyin_search"
-	dutils "pkg.deepin.io/lib/utils"
+	"github.com/linuxdeepin/dde-daemon/keybinding/util"
 )
 
 var logger *log.Logger
 
+func GetQtKeycodeMap() map[string]string {
+	var qtKeycodMap = map[string]string{
+		"messenger":           "Qt::Key_Messenger",             // XF86Messenger
+		"save":                "Qt::Key_Save",                  // XF86Save
+		"new":                 "Qt::Key_New",                   // XF86New
+		"wake-up":             "Qt::Key_WakeUp",                // XF86WakeUp
+		"audio-rewind":        "Qt::Key_AudioRewind",           // XF86AudioRewind
+		"audio-mute":          "Qt::Key_VolumeMute",            // XF86AudioMute
+		"mon-brightness-up":   "Qt::Key_MonBrightnessUp",       // XF86MonBrightnessUp
+		"wlan":                "Qt::Key_WLAN",                  // XF86WLAN
+		"audio-media":         "Qt::Key_AudioMedia",            // XF86AudioMedia
+		"reply":               "Qt::Key_Reply",                 // XF86Reply
+		"favorites":           "Qt::Key_Favorites",             // XF86Favorites
+		"audio-play":          "Qt::Key_MediaPlay",             // XF86AudioPlay
+		"audio-mic-mute":      "Qt::Key_MicMute",               // XF86AudioMicMute
+		"audio-pause":         "Qt::Key_MediaPause",            // XF86AudioPause
+		"audio-stop":          "Qt::Key_AudioStop",             // XF86AudioStop
+		"documents":           "Qt::Key_Documents",             // XF86Documents
+		"game":                "Qt::Key_Game",                  // XF86Game
+		"search":              "<Super><Shift>Touchpad Toggle", // XF86Search
+		"audio-record":        "Qt::Key_AudioRecord",           // XF86AudioRecord
+		"display":             "Qt::Key_Display",               // XF86Display
+		"reload":              "Qt::Key_Reload",                // XF86Reload
+		"explorer":            "Qt::Key_Explorer",              // XF86Explorer
+		"calculator":          "Qt::Key_Launch1",               // XF86Calculator
+		"calendar":            "Qt::Key_Calendar",              // XF86Calendar
+		"forward":             "Qt::Key_Forward",               // XF86Forward
+		"cut":                 "Qt::Key_Cut",                   // XF86Cut
+		"mon-brightness-down": "Qt::Key_MonBrightnessDown",     // XF86MonBrightnessDown
+		"copy":                "Qt::Key_Copy",                  // XF86Copy
+		"tools":               "Qt::Key_Tools",                 // XF86Tools
+		"audio-raise-volume":  "Qt::Key_VolumeUp",              // XF86AudioRaiseVolume
+		"media-close":         "Qt::Key_Close",                 // XF86Close
+		"www":                 "Qt::Key_WWW",                   // XF86WWW
+		"home-page":           "Qt::Key_HomePage",              // XF86HomePage
+		"sleep":               "Qt::Key_Sleep",                 // XF86Sleep
+		"audio-lower-volume":  "Qt::Key_VolumeDown",            // XF86AudioLowerVolume
+		"audio-prev":          "Qt::Key_MediaPrevious",         // XF86AudioPrev
+		"audio-next":          "Qt::Key_MediaNext",             // XF86AudioNext
+		"paste":               "Qt::Key_Paste",                 // XF86Paste
+		"open":                "Qt::Key_Open",                  // XF86Open
+		"send":                "Qt::Key_Send",                  // XF86Send
+		"my-computer":         "Qt::Key_MyComputer",            // XF86MyComputer
+		"mail":                "Qt::Key_MailForward",           // XF86Mail
+		"adjust-brightness":   "Qt::Key_BrightnessAdjust",      // XF86BrightnessAdjust
+		"log-off":             "Qt::Key_LogOff",                // XF86LogOff
+		"pictures":            "Qt::Key_Pictures",              // XF86Pictures
+		"terminal":            "Qt::Key_Terminal",              // XF86Terminal
+		"video":               "Qt::Key_Video",                 // XF86Video
+		"music":               "Qt::Key_Music",                 // XF86Music
+		"app-left":            "Qt::Key_ApplicationLeft",       // XF86ApplicationLeft
+		"app-right":           "Qt::Key_ApplicationRight",      // XF86ApplicationRight
+		"meeting":             "Qt::Key_Meeting",               // XF86Meeting
+		"switch-monitors":     "<Super>P",
+		"numlock":             "Qt::Key_NumLock",
+		"capslock":            "Qt::Key_CapsLock",
+	}
+	return qtKeycodMap
+}
+
 const (
 	SKLCtrlShift uint32 = 1 << iota
 	SKLAltShift
@@ -63,7 +125,7 @@ type KeyEventFunc func(ev *KeyEvent)
 type ShortcutManager struct {
 	conn         *x.Conn
 	dataConn     *x.Conn // conn for receive record event
-	daemonDaemon *daemon.Daemon
+	daemonDaemon daemon.Daemon
 
 	idShortcutMap     map[string]Shortcut
 	idShortcutMapMu   sync.Mutex
@@ -695,29 +757,6 @@ func (sm *ShortcutManager) handleXRecordKeyEvent(pressed bool, code uint8, state
 			if shortcut != nil && shortcut.GetType() == ShortcutTypeSystem &&
 				(strings.HasPrefix(shortcut.GetId(), "screenshot") ||
 					strings.HasPrefix(shortcut.GetId(), "deepin-screen-recorder")) {
-				//如果当前没有窗口被grab,则直接不用判断是否需要截图
-				if !isKbdAlreadyGrabbed(sm.conn) {
-					logger.Info("handleXRecordKeyEvent : no active window grabbed")
-					return
-				}
-				//先获取active应用pid, 传给dde-system-daemon解析
-				//判断是否是在 虚拟机/云平台 进行截图
-				pid, err := sm.getActiveWindowPid()
-				if err == nil {
-					isVirtualMachine, err := sm.isPidVirtualMachine(pid)
-					logger.Info("handleXRecordKeyEvent, isVirtualMachine:", isVirtualMachine)
-					//获取出错时，当做非虚拟机
-					if err != nil {
-						logger.Warning(err)
-					} else {
-						//true:表示当前是 虚拟机/云平台，要过滤掉
-						if isVirtualMachine {
-							logger.Info("current active window is virtual or cloud")
-							return
-						}
-					}
-				}
-
 				keyEvent := &KeyEvent{
 					Mods:     key.Mods,
 					Code:     key.Code,
@@ -854,50 +893,131 @@ func (sm *ShortcutManager) FindConflictingKeystroke(ks *Keystroke) (*Keystroke,
 	return nil, nil
 }
 
-func (sm *ShortcutManager) AddSystem(gsettings *gio.Settings, wmObj *wm.Wm) {
-	logger.Debug("AddSystem")
+func systemType() string {
+	kf := keyfile.NewKeyFile()
+	err := kf.LoadFromFile("/etc/os-version")
+	if err != nil {
+		fmt.Println("load version file failed, err: ", err)
+		return ""
+	}
+	typ, err := kf.GetString("Version", "EditionName")
+	if err != nil {
+		fmt.Println("get version type failed, err: ", err)
+		return ""
+	}
+
+	return typ
+}
+
+func arr2set(in []string) map[string]bool {
+	out := make(map[string]bool)
+	for _, v := range in {
+		out[v] = true
+	}
+
+	return out
+}
+
+func strvLower(in []string) []string {
+	out := make([]string, len(in))
+	for i, v := range in {
+		out[i] = strings.ToLower(v)
+	}
+	return out
+}
+
+// 检测一个系统快捷键是否配置为可用，true可用，false不可用
+func (sm *ShortcutManager) CheckSystem(gsPlatform, gsEnable *gio.Settings, id string) bool {
+	platformSet := arr2set(gsPlatform.ListKeys())
+	enableSet := arr2set(gsEnable.ListKeys())
+	sysType := strings.ToLower(systemType())
+
+	// 判断是否是支持的平台
+	if platformSet[id] {
+		plats := gsPlatform.GetStrv(id)
+
+		platSet := arr2set(strvLower(plats))
+		if !platSet["all"] && !platSet[sysType] {
+			return false
+		}
+	}
+
+	// 判断是否配置开启
+	if enableSet[id] && !gsEnable.GetBoolean(id) {
+		logger.Debugf("%s is disabled", id)
+		return false
+	}
+
+	return true
+}
+
+func (sm *ShortcutManager) AddSystemById(gsettings *gio.Settings, id string) {
+	shortcut := sm.GetByIdType(id, ShortcutTypeSystem)
+	if shortcut != nil {
+		logger.Debugf("%s is exist", id)
+		return
+	}
+
 	idNameMap := getSystemIdNameMap()
+	name := idNameMap[id]
+	if name == "" {
+		name = id
+	}
+
+	cmd := getSystemActionCmd(id)
+	if id == "terminal-quake" && strings.Contains(cmd, "deepin-terminal") {
+		termPath, _ := exec.LookPath("deepin-terminal")
+		if termPath == "" {
+			return
+		}
+	}
+
+	keystrokes := gsettings.GetStrv(id)
+	gs := NewGSettingsShortcut(gsettings, id, ShortcutTypeSystem, keystrokes, name)
+	sysShortcut := &SystemShortcut{
+		GSettingsShortcut: gs,
+		arg: &ActionExecCmdArg{
+			Cmd: cmd,
+		},
+	}
+	sm.addWithoutLock(sysShortcut)
+}
+
+func (sm *ShortcutManager) DelSystemById(id string) {
+	shortcut := sm.GetByIdType(id, ShortcutTypeSystem)
+	if shortcut == nil {
+		logger.Debugf("%s is not exist", id)
+		return
+	}
+	sm.Delete(shortcut)
+}
+
+func (sm *ShortcutManager) AddSystem(gsettings, gsPlatform, gsEnable *gio.Settings, wmObj wm.Wm) {
+	logger.Debug("AddSystem")
 	allow, err := wmObj.CompositingAllowSwitch().Get(0)
 	if err != nil {
 		logger.Warning(err)
 		allow = false
 	}
-	session := os.Getenv("XDG_SESSION_TYPE")
+	sessionType := os.Getenv("XDG_SESSION_TYPE")
 	for _, id := range gsettings.ListKeys() {
 		if id == "deepin-screen-recorder" || id == "wm-switcher" {
-			if !allow {
+			if !allow && id == "wm-switcher" {
 				logger.Debugf("com.deepin.wm.compositingAllowSwitch is false, filter %s", id)
 				continue
 			}
 
-			if strings.Contains(session, "wayland") {
-				logger.Debugf("XDG_SESSION_TYPE is %s, filter %s", session, id)
+			if strings.Contains(sessionType, "wayland") {
+				logger.Debugf("XDG_SESSION_TYPE is %s, filter %s", sessionType, id)
 				continue
 			}
 		}
 
-		name := idNameMap[id]
-		if name == "" {
-			name = id
+		if !sm.CheckSystem(gsPlatform, gsEnable, id) {
+			continue
 		}
 
-		cmd := getSystemActionCmd(id)
-		if id == "terminal-quake" && strings.Contains(cmd, "deepin-terminal") {
-			termPath, _ := exec.LookPath("deepin-terminal")
-			if termPath == "" {
-				continue
-			}
-		}
-
-		keystrokes := gsettings.GetStrv(id)
-		gs := NewGSettingsShortcut(gsettings, id, ShortcutTypeSystem, keystrokes, name)
-		sysShortcut := &SystemShortcut{
-			GSettingsShortcut: gs,
-			arg: &ActionExecCmdArg{
-				Cmd: cmd,
-			},
-		}
-		sm.addWithoutLock(sysShortcut)
+		sm.AddSystemById(gsettings, id)
 	}
 }
 
@@ -960,7 +1080,7 @@ func (sm *ShortcutManager) AddSpecial() {
 	sm.addWithoutLock(s0)
 }
 
-func (sm *ShortcutManager) AddKWin(wmObj *wm.Wm) {
+func (sm *ShortcutManager) AddKWin(wmObj wm.Wm) {
 	logger.Debug("AddKWin")
 	accels, err := util.GetAllKWinAccels(wmObj)
 	if err != nil {
@@ -986,6 +1106,141 @@ func (sm *ShortcutManager) AddKWin(wmObj *wm.Wm) {
 	}
 }
 
+func (sm *ShortcutManager) AddKWinForWayland(wmObj wm.Wm) {
+	logger.Debug("AddKWinForWayland")
+	accels, err := util.GetAllKWinAccels(wmObj)
+	if err != nil {
+		logger.Warning("failed to get all KWin accels:", err)
+		return
+	}
+	idNameMap := getWMIdNameMap()
+	for _, accel := range accels {
+		// 'preview-workspace' unsupported in wayland, so filter it
+		if accel.Id == "preview-workspace" || accel.Id == "color-picker" {
+			continue
+		}
+		name := idNameMap[accel.Id]
+		if name == "" {
+			name = getSystemIdNameMap()[accel.Id]
+			logger.Debug("action", accel.Id, name)
+		}
+		if name == "" {
+			name = accel.Id
+		}
+		keystrokes := accel.Keystrokes
+		if keystrokes == nil {
+			keystrokes = accel.DefaultKeystrokes
+		}
+		//Del
+		if accel.Id == "logout" {
+			for i := 0; i < len(keystrokes); i++ {
+				keystrokes[i] = strings.Replace(keystrokes[i], "Del", "Delete", 1)
+			}
+		}
+		//minimize
+		if accel.Id == "minimize" {
+			if accel.DefaultKeystrokes != nil {
+				keystrokes = accel.DefaultKeystrokes
+			}
+		}
+		//launcher
+		if accel.Id == "launcher" {
+			if keystrokes == nil {
+				keystrokes = append(keystrokes, "<Super_L>")
+			}
+		}
+		//system-monitor
+		if accel.Id == "system-monitor" {
+			for i := 0; i < len(keystrokes); i++ {
+				keystrokes[i] = strings.Replace(keystrokes[i], "Esc", "Escape", 1)
+			}
+		}
+		ks := newKWinShortcut(accel.Id, name, keystrokes, wmObj)
+		sm.addWithoutLock(ks)
+	}
+}
+
+func (sm *ShortcutManager) AddSystemToKwin(gsettings *gio.Settings, wmObj wm.Wm) {
+	logger.Debug("AddSystemToKwin")
+	idNameMap := getSystemIdNameMap()
+	for _, id := range gsettings.ListKeys() {
+		name := idNameMap[id]
+		if name == "" {
+			name = id
+		}
+
+		if id == "launcher" || id == "system_monitor" {
+			continue
+		}
+
+		accelJson, err := util.MarshalJSON(util.KWinAccel{
+			Id:         id,
+			Keystrokes: gsettings.GetStrv(id),
+		})
+
+		if id == "screenshot-window" {
+			accelJson = `{"Id":"screenshot-window","Accels":["SysReq"]}` //+ Alt+print对应kwin识别的键SysReq
+		}
+		if err != nil {
+			logger.Warning("failed to get json:", err)
+			continue
+		}
+		ok, err := wmObj.SetAccel(0, accelJson)
+		if !ok {
+			logger.Warning("failed to set KWin accels:", id, gsettings.GetStrv(id), err)
+		}
+	}
+}
+
+func (sm *ShortcutManager) AddMediaToKwin(gsettings *gio.Settings, wmObj wm.Wm) {
+	logger.Debug("AddMediaToKwin")
+	idNameMap := getMediaIdNameMap()
+	for _, id := range gsettings.ListKeys() {
+		name := idNameMap[id]
+		logger.Warning("+++ gsetting KWin accels ID , NANE:", id, name)
+
+		if name == "" {
+			name = id
+		}
+		if len(os.Getenv("WAYLAND_DISPLAY")) != 0 {
+			if name == "Capslock" || name == "Numlock" {
+				continue
+			}
+		}
+
+		accelJson, err := util.MarshalJSON(util.KWinAccel{
+			Id:         id,
+			Keystrokes: []string{GetQtKeycodeMap()[id]}, //gsettings.GetStrv(id),
+		})
+		if err != nil {
+			logger.Warning("failed to get json:", err)
+			continue
+		}
+
+		ok, err := wmObj.SetAccel(0, accelJson)
+		if !ok {
+			logger.Warning("failed to set KWin accels:", accelJson, err)
+		}
+	}
+}
+
+func (sm *ShortcutManager) AddSpecialToKwin(wmObj wm.Wm) {
+	accelJson, err := util.MarshalJSON(util.KWinAccel{
+		Id:         "switch-kbd-layout",
+		Keystrokes: []string{"<Super>space"},
+	})
+	if err != nil {
+		logger.Warning("failed to get json:", err)
+	}
+
+	ok, err := wmObj.SetAccel(0, accelJson)
+	if !ok {
+		logger.Warning("failed to set KWin accels:", accelJson, err)
+	}
+	logger.Warning("set sepical KWin accels:", accelJson)
+
+}
+
 func isZH() bool {
 	lang := gettext.QueryLang()
 	return strings.HasPrefix(lang, "zh")
