@@ -4,6 +4,7 @@ import (
 	"bytes"
 	"encoding/json"
 	"strings"
+	"os"
 
 	wm "github.com/linuxdeepin/go-dbus-factory/com.deepin.wm"
 )
@@ -49,11 +50,15 @@ func (kwa *KWinAccel) fix() {
 	kwa.DefaultKeystrokes = defaultKeystrokes
 }
 
-func GetAllKWinAccels(wm *wm.Wm) ([]KWinAccel, error) {
+func GetAllKWinAccels(wm wm.Wm) ([]KWinAccel, error) {
 	allJson, err := wm.GetAllAccels(0)
 	if err != nil {
 		return nil, err
 	}
+	sessionType := os.Getenv("XDG_SESSION_TYPE")
+	if strings.Contains(sessionType, "wayland") && strings.Contains(allJson, "SysReq") {
+		allJson = strings.Replace(allJson, "SysReq", "<Alt>Print", -1)
+	}
 
 	var result []KWinAccel
 	err = json.Unmarshal([]byte(allJson), &result)
