@@ -37,7 +37,7 @@ LC_TIME="zh_CN.UTF-8"
 
 func Test_GetLocale(t *testing.T) {
 	l, err := getLocaleFromFile("testdata/pam_environment")
-	assert.Nil(t, err)
+	assert.NoError(t, err)
 	assert.Equal(t, l, "zh_CN.UTF-8")
 
 	l = getCurrentUserLocale()
@@ -45,6 +45,6 @@ func Test_GetLocale(t *testing.T) {
 }
 
 func Test_WriteUserLocale(t *testing.T) {
-	assert.Nil(t, writeLocaleEnvFile("zh_CN.UTF-8", "testdata/pam"))
+	assert.Nil(t, writeLocaleEnvFile("zh_CN.UTF-8", "testdata/pam_environment", "testdata/pam"))
 	os.RemoveAll("testdata/pam")
 }
