@@ -22,23 +22,22 @@ package network
 import (
 	"errors"
 	"fmt"
-	"net"
 	"strings"
 	"time"
 
 	dbus "github.com/godbus/dbus"
+	"github.com/linuxdeepin/dde-daemon/network/nm"
 	mmdbus "github.com/linuxdeepin/go-dbus-factory/org.freedesktop.modemmanager1"
 	nmdbus "github.com/linuxdeepin/go-dbus-factory/org.freedesktop.networkmanager"
-	"pkg.deepin.io/dde/daemon/network/nm"
-	"pkg.deepin.io/lib/dbusutil"
+	"github.com/linuxdeepin/go-lib/dbusutil"
 )
 
 type device struct {
-	nmDev      *nmdbus.Device
-	mmDevModem *mmdbus.Modem
+	nmDev      nmdbus.Device
+	mmDevModem mmdbus.Modem
 	nmDevType  uint32
 	id         string
-	udi        string
+	Udi        string
 
 	Path          dbus.ObjectPath
 	State         uint32
@@ -70,8 +69,6 @@ type device struct {
 	MobileSignalQuality uint32
 
 	InterfaceFlags uint32
-
-	quitFlagsCheckChan chan struct{}
 }
 
 const (
@@ -120,7 +117,7 @@ func (m *Manager) adjustDeviceStatus() {
 
 		// 如果nmDevice在network启动之前就已经是密码待验证的状态，需要等待nm超时才能自动回连，导致wifi回连太慢
 		// 在这种情况下主动回连
-		stat, _ := nmDev.State().Get(0)
+		stat, _ := nmDev.Device().State().Get(0)
 		if stat == nm.NM_DEVICE_STATE_NEED_AUTH {
 			err := m.enableDevice(path, true, true)
 			if err != nil {
@@ -138,13 +135,13 @@ func (m *Manager) newDevice(devPath dbus.ObjectPath) (dev *device, err error) {
 
 	// ignore virtual network interfaces
 	if isVirtualDeviceIfc(nmDev) {
-		driver, _ := nmDev.Driver().Get(0)
+		driver, _ := nmDev.Device().Driver().Get(0)
 		err = fmt.Errorf("ignore virtual network interface which driver is %s %s", driver, devPath)
 		logger.Info(err)
 		return
 	}
 
-	devType, _ := nmDev.DeviceType().Get(0)
+	devType, _ := nmDev.Device().DeviceType().Get(0)
 	if !isDeviceTypeValid(devType) {
 		err = fmt.Errorf("ignore invalid device type %d", devType)
 		logger.Info(err)
@@ -156,8 +153,8 @@ func (m *Manager) newDevice(devPath dbus.ObjectPath) (dev *device, err error) {
 		nmDevType: devType,
 		Path:      nmDev.Path_(),
 	}
-	dev.udi, _ = nmDev.Udi().Get(0)
-	dev.Driver, _ = nmDev.Driver().Get(0)
+	dev.Udi, _ = nmDev.Device().Udi().Get(0)
+	dev.Driver, _ = nmDev.Device().Driver().Get(0)
 
 	dev.Vendor = nmGeneralGetDeviceDesc(devPath)
 	dev.UsbDevice = nmGeneralIsUsbDevice(devPath)
@@ -191,6 +188,22 @@ func (m *Manager) newDevice(devPath dbus.ObjectPath) (dev *device, err error) {
 			dev.HwAddress = dev.ClonedAddress
 		}
 
+		// wired device should always create non-tmp connections, nm will create tmp connection sometimes when device first plugin in
+		err = nmDev.Device().ActiveConnection().ConnectChanged(func(hasValue bool, value dbus.ObjectPath) {
+			// if has not value or value is not expected, ignore changes
+			if !hasValue || value == "/" || value == "" {
+				return
+			}
+			// try to get active connection
+			_, _, err = m.ensureWiredConnectionExists(devPath, true)
+			if err != nil {
+				logger.Warningf("ensure wired connection failed, err: %v", err)
+			}
+		})
+		if err != nil {
+			logger.Warningf("connect to ActivateConnection failed, err: %v", err)
+		}
+
 		if nmHasSystemSettingsModifyPermission() {
 			carrierChanged := func(hasValue, value bool) {
 				if !hasValue || !value {
@@ -234,12 +247,9 @@ func (m *Manager) newDevice(devPath dbus.ObjectPath) (dev *device, err error) {
 			dev.ActiveAp = value
 			m.updatePropDevices()
 
-			// Re-active connection if wireless 'ActiveAccessPoint' not equal active connection 'SpecificObject'
-			// such as wifi roaming, but the active connection state is activated
-			err := m.wirelessReActiveConnection(nmDev)
-			if err != nil {
-				logger.Warning("Failed to re-active connection:", err)
-			}
+			// when wifi is roaming, wpa and network-manager will deal this situation,
+			// dde dont need try to re active connection or may cause error connection in OPT env
+			// this case always means nm or wpa has bug, fix nm or wpa is better
 		})
 		if err != nil {
 			logger.Warning(err)
@@ -267,6 +277,7 @@ func (m *Manager) newDevice(devPath dbus.ObjectPath) (dev *device, err error) {
 				return
 			}
 
+			m.accessPointsLock.Lock()
 			shouldRemove := make([]dbus.ObjectPath, 0, len(m.accessPoints[devPath]))
 			for _, a := range m.accessPoints[devPath] {
 				var found bool
@@ -297,7 +308,6 @@ func (m *Manager) newDevice(devPath dbus.ObjectPath) (dev *device, err error) {
 				}
 			}
 
-			m.accessPointsLock.Lock()
 			for _, a := range shouldRemove {
 				m.removeAccessPoint(devPath, a)
 			}
@@ -305,6 +315,10 @@ func (m *Manager) newDevice(devPath dbus.ObjectPath) (dev *device, err error) {
 			for _, a := range shouldAdd {
 				m.addAccessPoint(devPath, a)
 			}
+
+			m.PropsMu.Lock()
+			m.updatePropWirelessAccessPoints()
+			m.PropsMu.Unlock()
 			m.accessPointsLock.Unlock()
 		})
 		if err != nil {
@@ -331,12 +345,12 @@ func (m *Manager) newDevice(devPath dbus.ObjectPath) (dev *device, err error) {
 			time.Sleep(3 * time.Second)
 			nmSetDeviceAutoconnect(dev.Path, false)
 		}()
-		if mmDevModem, err := mmNewModem(dbus.ObjectPath(dev.udi)); err == nil {
+		if mmDevModem, err := mmNewModem(dbus.ObjectPath(dev.Udi)); err == nil {
 			mmDevModem.InitSignalExt(m.sysSigLoop, true)
 			dev.mmDevModem = mmDevModem
 
 			// connect properties
-			err = dev.mmDevModem.AccessTechnologies().ConnectChanged(func(hasValue bool,
+			err = dev.mmDevModem.Modem().AccessTechnologies().ConnectChanged(func(hasValue bool,
 				value uint32) {
 				if !m.isDeviceExists(devPath) {
 					return
@@ -352,10 +366,10 @@ func (m *Manager) newDevice(devPath dbus.ObjectPath) (dev *device, err error) {
 			if err != nil {
 				logger.Warning(err)
 			}
-			accessTech, _ := mmDevModem.AccessTechnologies().Get(0)
+			accessTech, _ := mmDevModem.Modem().AccessTechnologies().Get(0)
 			dev.MobileNetworkType = mmDoGetModemMobileNetworkType(accessTech)
 
-			err = dev.mmDevModem.SignalQuality().ConnectChanged(func(hasValue bool,
+			err = dev.mmDevModem.Modem().SignalQuality().ConnectChanged(func(hasValue bool,
 				value mmdbus.ModemSignalQuality) {
 				if !m.isDeviceExists(devPath) {
 					return
@@ -377,7 +391,7 @@ func (m *Manager) newDevice(devPath dbus.ObjectPath) (dev *device, err error) {
 	}
 
 	// connect signals
-	_, err = dev.nmDev.ConnectStateChanged(func(newState uint32, oldState uint32, reason uint32) {
+	_, err = dev.nmDev.Device().ConnectStateChanged(func(newState uint32, oldState uint32, reason uint32) {
 		logger.Debugf("device %s state changed, %d => %d, reason[%d] %s",
 			string(devPath), oldState, newState, reason, deviceErrorTable[reason])
 
@@ -400,7 +414,7 @@ func (m *Manager) newDevice(devPath dbus.ObjectPath) (dev *device, err error) {
 		logger.Warning(err)
 	}
 
-	err = dev.nmDev.Interface().ConnectChanged(func(hasValue bool, value string) {
+	err = dev.nmDev.Device().Interface().ConnectChanged(func(hasValue bool, value string) {
 		if !hasValue {
 			return
 		}
@@ -414,7 +428,7 @@ func (m *Manager) newDevice(devPath dbus.ObjectPath) (dev *device, err error) {
 		logger.Warning(err)
 	}
 
-	err = dev.nmDev.Managed().ConnectChanged(func(hasValue bool, value bool) {
+	err = dev.nmDev.Device().Managed().ConnectChanged(func(hasValue bool, value bool) {
 		if !hasValue {
 			return
 		}
@@ -428,66 +442,50 @@ func (m *Manager) newDevice(devPath dbus.ObjectPath) (dev *device, err error) {
 		logger.Warning(err)
 	}
 
-	dev.State, _ = nmDev.State().Get(0)
-	dev.Interface, _ = nmDev.Interface().Get(0)
+	dev.State, _ = nmDev.Device().State().Get(0)
+	dev.Interface, _ = nmDev.Device().Interface().Get(0)
 	// get device enable state from system network
 	enabled, _ := m.sysNetwork.IsDeviceEnabled(0, dev.Interface)
 	// adjust device enable state
 	// due to script in pre-up and pre-down, device will be always set as true or false,
-	// in this situation, the config kept in local file is not exact, config need be adjusted
-	if dev.State > nm.NM_DEVICE_STATE_DISCONNECTED && !enabled {
-		_, err = m.sysNetwork.EnableDevice(0, dev.Interface, true)
+	// in this situation, the config kept in local file is not exact, device state should be adjust
+	if isDeviceStateInActivating(dev.State) && !enabled {
+		err = dev.nmDev.Device().Disconnect(0)
 		if err != nil {
-			logger.Warningf("set device enable failed, err: %v", err)
+			logger.Warningf("device disconnected failed, err: %v", err)
+		}
+	} else if enabled {
+		err = dev.nmDev.Device().Autoconnect().Set(0, true)
+		if err != nil {
+			logger.Warningf("init set auto-connect state failed, err: %v", err)
 		}
 	}
 	dev.Managed = nmGeneralIsDeviceManaged(devPath)
 
-	// TODO: NetworkManager 升级 1.22 后，直接使用 NetworkManager 的 InterfaceFlags 属性
-	dev.InterfaceFlags = m.getInterfaceFlags(dev)
-	ticker := time.NewTicker(1 * time.Second)
-	dev.quitFlagsCheckChan = make(chan struct{})
-	go func() {
-		for {
-			select {
-			case <-ticker.C:
-				newFlags := m.getInterfaceFlags(dev)
-				if dev.InterfaceFlags != newFlags {
-					dev.InterfaceFlags = newFlags
-
-					m.devicesLock.Lock()
-					m.updatePropDevices()
-					m.devicesLock.Unlock()
-				}
-
-			case <-dev.quitFlagsCheckChan:
-				ticker.Stop()
-				return
-			}
+	// get interface-flags
+	dev.InterfaceFlags, err = nmDev.Device().InterfaceFlags().Get(0)
+	if err != nil {
+		logger.Warningf("get interface-flags failed, err: %v", err)
+	} else {
+		logger.Debugf("get interface-flags success, flags: %v", dev.InterfaceFlags)
+	}
+	// monitor interface-flags changed signal
+	err = dev.nmDev.Device().InterfaceFlags().ConnectChanged(func(hasValue bool, value uint32) {
+		if !hasValue {
+			return
 		}
-	}()
+		logger.Debugf("interface-flags changed, flags: %v", value)
+		dev.InterfaceFlags = value
+		m.updatePropDevices()
+	})
+	if err != nil {
+		logger.Warningf("connected interface-flags failed, err: %v", err)
+	}
 
 	return
 }
 
-func (m *Manager) getInterfaceFlags(dev *device) uint32 {
-	interfaceInfo, err := net.InterfaceByName(dev.Interface)
-	if err != nil {
-		logger.Warning("failed to get interface info:", err)
-		return 0
-	}
-
-	var flags uint32
-	if interfaceInfo.Flags&net.FlagUp != 0 {
-		flags |= nmInterfaceFlagUP
-	}
-
-	return flags
-}
-
 func (m *Manager) destroyDevice(dev *device) {
-	close(dev.quitFlagsCheckChan)
-
 	// destroy object to reset all property connects
 	if dev.mmDevModem != nil {
 		mmDestroyModem(dev.mmDevModem)
@@ -519,6 +517,7 @@ func (m *Manager) addDevice(devPath dbus.ObjectPath) {
 	if err != nil {
 		return
 	}
+
 	logger.Debug("add device", devPath)
 	devType := getCustomDeviceType(dev.nmDevType)
 	m.devices[devType] = append(m.devices[devType], dev)
@@ -588,18 +587,40 @@ func (m *Manager) enableDevice(devPath dbus.ObjectPath, enabled bool, activate b
 	if err != nil {
 		return
 	}
+	logger.Debugf("dev %v, enabled: %v, activate: %v", devPath, enabled, activate)
 	// check if need activate connection
-	if enabled && activate {
-		var uuid string
-		uuid, err = nmGetConnectionUuid(cpath)
-		if err != nil {
-			return
+	if enabled {
+		if cpath != "/" && activate {
+			var uuid string
+			uuid, err = nmGetConnectionUuid(cpath)
+			if err != nil {
+				return
+			}
+			_, err = m.activateConnection(uuid, devPath)
+			if err != nil {
+				logger.Debug("failed to activate a connection")
+				return
+			}
+			logger.Debugf("active connection success, dev: %v, uuid: %v", devPath, uuid)
 		}
-		_, err = m.activateConnection(uuid, devPath)
+
+		// activate connection first, then set auto-connect state,
+		// in case, auto-connect is set, nm cancel block connect and try to auto-activating
+		logger.Debugf("begin to set device auto-connect state, dev: %v", devPath)
+		nmDev, err := nmNewDevice(devPath)
 		if err != nil {
-			logger.Debug("failed to activate a connection")
-			return
+			logger.Warningf("create device failed, err: %v", err)
+			return err
 		}
+		// set auto-connect state,
+		// when device is enabled, set auto-connect as true
+		// when disconnect, auto-connect is set to false automatic
+		err = nmDev.Device().Autoconnect().Set(0, true)
+		if err != nil {
+			logger.Warningf("set device auto-connect failed, err: %v", err)
+			return err
+		}
+		logger.Debugf("device set auto-connect success, dev: %v", devPath)
 	}
 
 	// set enable device state
@@ -677,20 +698,20 @@ func (m *Manager) listDeviceConnections(devPath dbus.ObjectPath) ([]dbus.ObjectP
 
 	// ignore virtual network interfaces
 	if isVirtualDeviceIfc(nmDev) {
-		driver, _ := nmDev.Driver().Get(0)
+		driver, _ := nmDev.Device().Driver().Get(0)
 		err = fmt.Errorf("ignore virtual network interface which driver is %s %s", driver, devPath)
 		logger.Info(err)
 		return nil, err
 	}
 
-	devType, _ := nmDev.DeviceType().Get(0)
+	devType, _ := nmDev.Device().DeviceType().Get(0)
 	if !isDeviceTypeValid(devType) {
 		err = fmt.Errorf("ignore invalid device type %d", devType)
 		logger.Info(err)
 		return nil, err
 	}
 
-	availableConnections, _ := nmDev.AvailableConnections().Get(0)
+	availableConnections, _ := nmDev.Device().AvailableConnections().Get(0)
 	return availableConnections, nil
 }
 
@@ -700,7 +721,7 @@ func (m *Manager) RequestWirelessScan() *dbus.Error {
 	defer m.devicesLock.Unlock()
 	if devices, ok := m.devices[deviceWifi]; ok {
 		for _, dev := range devices {
-			err := dev.nmDev.RequestScan(0, nil)
+			err := dev.nmDev.Wireless().RequestScan(0, nil)
 			if err != nil {
 				logger.Debug(err)
 			}
@@ -709,7 +730,7 @@ func (m *Manager) RequestWirelessScan() *dbus.Error {
 	return nil
 }
 
-func (m *Manager) wirelessReActiveConnection(nmDev *nmdbus.Device) error {
+func (m *Manager) wirelessReActiveConnection(nmDev nmdbus.Device) error {
 	wireless := nmDev.Wireless()
 	apPath, err := wireless.ActiveAccessPoint().Get(0)
 	if err != nil {
@@ -719,7 +740,7 @@ func (m *Manager) wirelessReActiveConnection(nmDev *nmdbus.Device) error {
 		logger.Debug("Invalid active access point path:", nmDev.Path_())
 		return nil
 	}
-	connPath, err := nmDev.ActiveConnection().Get(0)
+	connPath, err := nmDev.Device().ActiveConnection().Get(0)
 	if err != nil {
 		return err
 	}
@@ -777,15 +798,12 @@ func (m *Manager) checkGatewayConnectivity(ipPath dbus.ObjectPath) bool {
 		return false
 	}
 
-	addr, mask, gateways, domains := nmGetIp4ConfigInfo(ipPath)
-	logger.Debugf("The active connection ip4 info: address(%s), mask(%s), gateways(%v), domains(%v)",
-		addr, mask, gateways, domains)
+	ip4Info := nmGetIp4ConfigInfo(ipPath)
+	logger.Debugf("The active connection ip4 info: %v",
+		ip4Info)
 	// check whether the gateway is connected by ping
-	for _, gw := range gateways {
-		if len(gw) == 0 {
-			continue
-		}
-		if !m.doPing(gw, 3) {
+	if len(ip4Info.Gateway) != 0 {
+		if !m.doPing(ip4Info.Gateway, 3) {
 			return false
 		}
 	}
