@@ -24,10 +24,10 @@ import (
 	"sync"
 
 	dbus "github.com/godbus/dbus"
+	"github.com/linuxdeepin/dde-daemon/network/nm"
 	nmdbus "github.com/linuxdeepin/go-dbus-factory/org.freedesktop.networkmanager"
-	"pkg.deepin.io/dde/daemon/network/nm"
-	"pkg.deepin.io/lib/dbusutil"
-	. "pkg.deepin.io/lib/gettext"
+	"github.com/linuxdeepin/go-lib/dbusutil"
+	. "github.com/linuxdeepin/go-lib/gettext"
 )
 
 var vpnErrorTable = make(map[uint32]string)
@@ -130,7 +130,7 @@ type stateHandler struct {
 }
 
 type deviceStateInfo struct {
-	nmDev          *nmdbus.Device
+	nmDev          nmdbus.Device
 	enabled        bool
 	devUdi         string
 	devType        uint32
@@ -200,7 +200,12 @@ func (sh *stateHandler) watch(path dbus.ObjectPath) {
 		return
 	}
 
-	deviceType, _ := nmDev.DeviceType().Get(0)
+	// dont notify message when virtual device state changed
+	if isVirtualDeviceIfc(nmDev) {
+		return
+	}
+
+	deviceType, _ := nmDev.Device().DeviceType().Get(0)
 	if !isDeviceTypeValid(deviceType) {
 		return
 	}
@@ -209,7 +214,7 @@ func (sh *stateHandler) watch(path dbus.ObjectPath) {
 	defer sh.locker.Unlock()
 	sh.devices[path] = &deviceStateInfo{nmDev: nmDev}
 	sh.devices[path].devType = deviceType
-	sh.devices[path].devUdi, _ = nmDev.Udi().Get(0)
+	sh.devices[path].devUdi, _ = nmDev.Device().Udi().Get(0)
 	enabled, err := sh.m.sysNetwork.IsDeviceEnabled(0, string(path))
 	if err == nil {
 		sh.devices[path].enabled = enabled
@@ -225,19 +230,33 @@ func (sh *stateHandler) watch(path dbus.ObjectPath) {
 
 	// connect signals
 	nmDev.InitSignalExt(sh.sysSigLoop, true)
-	_, err = nmDev.ConnectStateChanged(func(newState, oldState, reason uint32) {
-		globalSessionActive = sh.m.isSessionActive()
-
+	_, err = nmDev.Device().ConnectStateChanged(func(newState, oldState, reason uint32) {
+		var id string
+		sh.m.activeConnectionsLock.Lock()
+		for _, ac := range sh.m.activeConnections {
+			// search dev
+			for _, dev := range ac.Devices {
+				// check if type is equal
+				if dev == path {
+					id = ac.Id
+					break
+				}
+			}
+		}
+		sh.m.activeConnectionsLock.Unlock()
 		logger.Debugf("device state changed, %d => %d, reason[%d] %s", oldState, newState, reason, deviceErrorTable[reason])
 		sh.locker.Lock()
 		defer sh.locker.Unlock()
+		// update id here
+		if id != "" && id != "/" {
+			sh.devices[path].aconnId = id
+		}
 		if data, err := nmGetDeviceActiveConnectionData(path); err == nil {
-			// update active connection id and type if exists
-			sh.devices[path].aconnId = getSettingConnectionId(data)
+			// update active connection and type if exists
 			sh.devices[path].connectionType = getCustomConnectionType(data)
 		}
 		dsi, ok := sh.devices[path]
-		if !ok {
+		if !ok || (sh.devices[path].aconnId == "") {
 			// the device already been removed
 			return
 		}
