@@ -27,11 +27,11 @@ import (
 	"strings"
 	"time"
 
-	dbus "github.com/godbus/dbus"
+	"github.com/godbus/dbus"
+	"github.com/linuxdeepin/dde-daemon/network/nm"
 	nmdbus "github.com/linuxdeepin/go-dbus-factory/org.freedesktop.networkmanager"
-	"pkg.deepin.io/dde/daemon/network/nm"
-	"pkg.deepin.io/lib/dbusutil/proxy"
-	. "pkg.deepin.io/lib/gettext"
+	"github.com/linuxdeepin/go-lib/dbusutil/proxy"
+	. "github.com/linuxdeepin/go-lib/gettext"
 )
 
 // Wrapper NetworkManger dbus methods to hide
@@ -124,7 +124,8 @@ func isInDeviceWhitelist(filename string, ifc string) bool {
 
 }
 
-func isVirtualDeviceIfc(dev *nmdbus.Device) bool {
+func isVirtualDeviceIfc(d nmdbus.Device) bool {
+	dev := d.Device()
 	driver, _ := dev.Driver().Get(0)
 
 	//// workaround for huawei pangu
@@ -134,7 +135,7 @@ func isVirtualDeviceIfc(dev *nmdbus.Device) bool {
 	}
 
 	switch driver {
-	case "dummy", "veth", "vboxnet", "vmnet":
+	case "dummy", "veth", "vboxnet", "vmnet", "bridge":
 		return true
 	case "unknown", "vmxnet", "vmxnet2", "vmxnet3":
 		// sometimes we could not get vmnet dirver name, so check the
@@ -156,15 +157,13 @@ func nmGeneralGetDeviceHwAddr(devPath dbus.ObjectPath, perm bool) (hwAddr string
 		return
 	}
 
-	devType, _ := dev.DeviceType().Get(0)
+	devType, _ := dev.Device().DeviceType().Get(0)
 	switch devType {
 	case nm.NM_DEVICE_TYPE_ETHERNET:
 		devWired := dev.Wired()
 		hwAddr = ""
 		if perm {
-			hwAddr, err = devWired.PermHwAddress().Get(0)
-			hwAddr = strings.ToUpper(hwAddr)
-			return
+			hwAddr, _ = devWired.PermHwAddress().Get(0)
 		}
 		if hwAddr == "" {
 			// may get PermHwAddress failed under NetworkManager 1.4.1
@@ -219,10 +218,11 @@ func nmGeneralGetDeviceHwAddr(devPath dbus.ObjectPath, perm bool) (hwAddr string
 
 func nmGeneralGetDeviceIdentifier(devPath dbus.ObjectPath) (devId string, err error) {
 	// get device unique identifier, use hardware address if exists
-	dev, err := nmNewDevice(devPath)
+	d, err := nmNewDevice(devPath)
 	if err != nil {
 		return
 	}
+	dev := d.Device()
 
 	devType, _ := dev.DeviceType().Get(0)
 	switch devType {
@@ -233,7 +233,9 @@ func nmGeneralGetDeviceIdentifier(devPath dbus.ObjectPath) (devId string, err er
 		err = fmt.Errorf("could not get adsl device identifier now")
 		logger.Error(err)
 	case nm.NM_DEVICE_TYPE_ETHERNET:
-		devId, err = nmGeneralGetDeviceHwAddr(devPath, true)
+		// some device the 'hw_addr_perm' unset by driver, so use 'hw_addr' as id
+		// PMS Bug ID: 16704
+		devId, err = nmGeneralGetDeviceHwAddr(devPath, false)
 	default:
 		devId, err = nmGeneralGetDeviceHwAddr(devPath, true)
 	}
@@ -250,16 +252,13 @@ func nmGeneralGetDeviceUniqueUuid(devPath dbus.ObjectPath) (uuid string) {
 	return strToUuid(devId)
 }
 
-// get device network speed (Mb/s)
-func nmGeneralGetDeviceSpeed(devPath dbus.ObjectPath) (speedStr string) {
-	speed := uint32(0)
-	speedStr = Tr("Unknown")
+func nmGeneralGetDeviceSpeedNumeric(devPath dbus.ObjectPath) (speed uint32) {
 	dev, err := nmNewDevice(devPath)
 	if err != nil {
 		return
 	}
 
-	t, _ := dev.DeviceType().Get(0)
+	t, _ := dev.Device().DeviceType().Get(0)
 	switch t {
 	case nm.NM_DEVICE_TYPE_ETHERNET:
 		devWired := dev.Wired()
@@ -278,8 +277,22 @@ func nmGeneralGetDeviceSpeed(devPath dbus.ObjectPath) (speedStr string) {
 		speed = uint32(math.Trunc((float64(bitRate)/1000.0 + 0.5) * 10 / 10))
 	case nm.NM_DEVICE_TYPE_MODEM:
 		// TODO: getting device speed for modem device
+	case nm.NM_DEVICE_TYPE_BOND:
+		if ifc, err := dev.Device().Interface().Get(0); err == nil {
+			speed = getEthtoolCmdSpeedCgo(ifc)
+		}
+
 	default: // ignore speed for other device types
 	}
+
+	return
+}
+
+// get device network speed (Mb/s)
+func nmGeneralGetDeviceSpeed(devPath dbus.ObjectPath) (speedStr string) {
+	speedStr = Tr("Unknown")
+
+	speed := nmGeneralGetDeviceSpeedNumeric(devPath)
 	if speed != 0 {
 		speedStr = fmt.Sprintf("%d Mb/s", speed)
 	}
@@ -287,30 +300,25 @@ func nmGeneralGetDeviceSpeed(devPath dbus.ObjectPath) (speedStr string) {
 }
 
 func nmGeneralIsDeviceManaged(devPath dbus.ObjectPath) bool {
-	dev, err := nmNewDevice(devPath)
+	d, err := nmNewDevice(devPath)
 	if err != nil {
 		return false
 	}
+	dev := d.Device()
 
-	state, _ := dev.State().Get(0)
-	if !isDeviceStateManaged(state) {
-		return false
+	state, err := dev.State().Get(0)
+	if err != nil {
+		logger.Warning(err)
 	}
-	devType, _ := dev.DeviceType().Get(0)
-	switch devType {
-	case nm.NM_DEVICE_TYPE_WIFI:
-		if !nmGetWirelessHardwareEnabled() {
-			return false
-		}
-	}
-	return true
+	return isDeviceStateManaged(state)
 }
 
 func nmGeneralGetDeviceSysPath(devPath dbus.ObjectPath) (sysPath string, err error) {
-	dev, err := nmNewDevice(devPath)
+	d, err := nmNewDevice(devPath)
 	if err != nil {
 		return
 	}
+	dev := d.Device()
 
 	deviceType, _ := dev.DeviceType().Get(0)
 	devUdi, _ := dev.Udi().Get(0)
@@ -360,7 +368,7 @@ func nmGeneralIsUsbDevice(devPath dbus.ObjectPath) bool {
 }
 
 // New network manager objects
-func nmNewManager() (m *nmdbus.Manager, err error) {
+func nmNewManager() (m nmdbus.Manager, err error) {
 	systemBus, err := dbus.SystemBus()
 	if err != nil {
 		return nil, err
@@ -368,7 +376,7 @@ func nmNewManager() (m *nmdbus.Manager, err error) {
 	m = nmdbus.NewManager(systemBus)
 	return
 }
-func nmNewDevice(devPath dbus.ObjectPath) (dev *nmdbus.Device, err error) {
+func nmNewDevice(devPath dbus.ObjectPath) (dev nmdbus.Device, err error) {
 	systemBus, err := dbus.SystemBus()
 	if err != nil {
 		return nil, err
@@ -381,7 +389,7 @@ func nmNewDevice(devPath dbus.ObjectPath) (dev *nmdbus.Device, err error) {
 	return
 }
 
-func nmNewAccessPoint(apPath dbus.ObjectPath) (ap *nmdbus.AccessPoint, err error) {
+func nmNewAccessPoint(apPath dbus.ObjectPath) (ap nmdbus.AccessPoint, err error) {
 	systemBus, err := dbus.SystemBus()
 	if err != nil {
 		return
@@ -393,7 +401,7 @@ func nmNewAccessPoint(apPath dbus.ObjectPath) (ap *nmdbus.AccessPoint, err error
 	}
 	return
 }
-func nmNewActiveConnection(apath dbus.ObjectPath) (aconn *nmdbus.ActiveConnection, err error) {
+func nmNewActiveConnection(apath dbus.ObjectPath) (aconn nmdbus.ActiveConnection, err error) {
 	systemBus, err := dbus.SystemBus()
 	if err != nil {
 		return
@@ -406,7 +414,7 @@ func nmNewActiveConnection(apath dbus.ObjectPath) (aconn *nmdbus.ActiveConnectio
 	return
 }
 
-func nmNewIP4Config(path dbus.ObjectPath) (ip4config *nmdbus.IP4Config, err error) {
+func nmNewIP4Config(path dbus.ObjectPath) (ip4config nmdbus.IP4Config, err error) {
 	systemBus, err := dbus.SystemBus()
 	if err != nil {
 		return
@@ -418,7 +426,7 @@ func nmNewIP4Config(path dbus.ObjectPath) (ip4config *nmdbus.IP4Config, err erro
 	}
 	return
 }
-func nmNewIP6Config(path dbus.ObjectPath) (ip6config *nmdbus.IP6Config, err error) {
+func nmNewIP6Config(path dbus.ObjectPath) (ip6config nmdbus.IP6Config, err error) {
 	systemBus, err := dbus.SystemBus()
 	if err != nil {
 		return
@@ -431,7 +439,7 @@ func nmNewIP6Config(path dbus.ObjectPath) (ip6config *nmdbus.IP6Config, err erro
 	}
 	return
 }
-func nmNewSettingsConnection(cpath dbus.ObjectPath) (conn *nmdbus.ConnectionSettings, err error) {
+func nmNewSettingsConnection(cpath dbus.ObjectPath) (conn nmdbus.ConnectionSettings, err error) {
 	systemBus, err := dbus.SystemBus()
 	if err != nil {
 		return
@@ -444,7 +452,7 @@ func nmNewSettingsConnection(cpath dbus.ObjectPath) (conn *nmdbus.ConnectionSett
 	return
 }
 
-func nmDestroyDevice(dev *nmdbus.Device) {
+func nmDestroyDevice(dev nmdbus.Device) {
 	if dev == nil {
 		logger.Error("Device to destroy is nil")
 		return
@@ -452,7 +460,7 @@ func nmDestroyDevice(dev *nmdbus.Device) {
 	dev.RemoveHandler(proxy.RemoveAllHandlers)
 }
 
-func nmDestroyAccessPoint(ap *nmdbus.AccessPoint) {
+func nmDestroyAccessPoint(ap nmdbus.AccessPoint) {
 	if ap == nil {
 		logger.Error("AccessPoint to destroy is nil")
 		return
@@ -460,7 +468,7 @@ func nmDestroyAccessPoint(ap *nmdbus.AccessPoint) {
 	ap.RemoveHandler(proxy.RemoveAllHandlers)
 }
 
-func nmDestroySettingsConnection(conn *nmdbus.ConnectionSettings) {
+func nmDestroySettingsConnection(conn nmdbus.ConnectionSettings) {
 	if conn == nil {
 		logger.Error("SettingsConnection to destroy is nil")
 		return
@@ -510,11 +518,12 @@ func nmGetDevices() (devPaths []dbus.ObjectPath) {
 }
 
 func nmGetDeviceInterface(devPath dbus.ObjectPath) (devInterface string) {
-	dev, err := nmNewDevice(devPath)
+	d, err := nmNewDevice(devPath)
 	if err != nil {
 		return
 	}
 
+	dev := d.Device()
 	devInterface, _ = dev.Interface().Get(0)
 	return
 }
@@ -613,7 +622,7 @@ func nmGetAccessPoints(devPath dbus.ObjectPath) (apPaths []dbus.ObjectPath) {
 }
 
 func nmGetManagerState() (state uint32) {
-	state, _ = nmManager.State().Get(0)
+	state, _ = nmManager.PropState().Get(0)
 	return
 }
 
@@ -667,6 +676,15 @@ func nmGetConnectionId(cpath dbus.ObjectPath) (id string) {
 	return
 }
 
+func nmGetConnectionVpnType(cpath dbus.ObjectPath) (service string) {
+	data, err := nmGetConnectionData(cpath)
+	if err != nil {
+		return
+	}
+	service = getSettingVpnServiceType(data)
+	return
+}
+
 func nmGetConnectionUuid(cpath dbus.ObjectPath) (uuid string, err error) {
 	data, err := nmGetConnectionData(cpath)
 	if err != nil {
@@ -677,10 +695,25 @@ func nmGetConnectionUuid(cpath dbus.ObjectPath) (uuid string, err error) {
 }
 
 func nmGetConnectionList() (connections []dbus.ObjectPath) {
-	connections, err := nmSettings.ListConnections(0)
-	if err != nil {
-		logger.Error(err)
-		return
+	var err error
+	retry := 0
+	for {
+		if retry > 3 {
+			logger.Warningf("has retry to get connection max time, err: %v", err)
+			break
+		}
+		// get connections list
+		connections, err = nmSettings.ListConnections(0)
+		if err != nil {
+			// if failed, just retry to get 2 times, so it is debus log here
+			logger.Debugf("get connection list failed, retry to get, err: %v", err)
+			retry++
+			time.Sleep(2 * time.Second)
+			continue
+		}
+		// get connection success
+		logger.Debugf("get all connection success, %v", connections)
+		break
 	}
 	return
 }
@@ -691,12 +724,13 @@ func nmGetConnectionByUuid(uuid string) (cpath dbus.ObjectPath, err error) {
 }
 
 func isWiredDevice(devPath dbus.ObjectPath) bool {
-	device, err := nmNewDevice(devPath)
+	d, err := nmNewDevice(devPath)
 	if err != nil {
 		return false
 	}
 
-	deviceType, _ := device.DeviceType().Get(0)
+	dev := d.Device()
+	deviceType, _ := dev.DeviceType().Get(0)
 	return deviceType == nm.NM_DEVICE_TYPE_ETHERNET
 }
 
@@ -735,64 +769,78 @@ func nmAddConnection(data connectionData) (cpath dbus.ObjectPath, err error) {
 	return
 }
 
-func nmAddConnectionUnsaved(data connectionData) (cpath dbus.ObjectPath, err error) {
-	cpath, err = nmSettings.AddConnectionUnsaved(0, data)
-	if err != nil {
-		logger.Error(err)
+func parseAddressDataProp(addressData []map[string]dbus.Variant) (addresses []addressDataItem) {
+	addresses = make([]addressDataItem, 0, len(addressData))
+	for _, v := range addressData {
+		a := v["address"].Value()
+		addr, ok := a.(string)
+		if !ok {
+			logger.Warningf("failed to convert address %v to string", a)
+		}
+
+		p := v["prefix"].Value()
+		prefix, ok := p.(uint32)
+		if !ok {
+			logger.Warningf("failed to convert prefix %v to uint", p)
+		}
+
+		addresses = append(addresses, addressDataItem{
+			Address: addr,
+			Prefix:  prefix,
+		})
 	}
-	return
+
+	return addresses
 }
 
-func nmGetIp4ConfigInfo(path dbus.ObjectPath) (address, mask string, gateways, nameServers []string) {
-	address = "0.0.0.0"
-	mask = "0.0.0.0"
+func nmGetIp4ConfigInfo(path dbus.ObjectPath) (ip4Data ipv4Info) {
 	ip4config, err := nmNewIP4Config(path)
 	if err != nil {
 		return
 	}
-	addressProp, _ := ip4config.Addresses().Get(0)
 
-	ipv4Addresses := wrapIpv4Addresses(addressProp)
-	if len(ipv4Addresses) > 0 {
-		address = ipv4Addresses[0].Address
-		mask = ipv4Addresses[0].Mask
-	}
-	for _, address := range ipv4Addresses {
-		gateways = append(gateways, address.Gateway)
+	addressDataProp, _ := ip4config.AddressData().Get(0)
+	ip4Data.Addresses = parseAddressDataProp(addressDataProp)
+	if len(ip4Data.Addresses) == 0 {
+		ip4Data.Addresses = []addressDataItem{
+			emptyIPv4,
+		}
 	}
 
-	nameServersProp, _ := ip4config.Nameservers().Get(0)
-	nameServers = wrapIpv4Dns(nameServersProp)
+	ip4Data.Gateway, _ = ip4config.Gateway().Get(0)
+
+	nameserversProp, _ := ip4config.Nameservers().Get(0)
+	ip4Data.Nameservers = wrapIpv4Dns(nameserversProp)
 	return
 }
 
-func nmGetIp6ConfigInfo(path dbus.ObjectPath) (address, prefix string, gateways, nameServers []string) {
-	address = "0::0"
-	prefix = "0"
+func nmGetIp6ConfigInfo(path dbus.ObjectPath) (ip6Data ipv6Info) {
 	ip6config, err := nmNewIP6Config(path)
 	if err != nil {
 		return
 	}
 
-	addressProp, _ := ip6config.Addresses().Get(0)
-	gateway, _ := ip6config.Gateway().Get(0)
-	gateways = append(gateways, gateway)
-	ipv6Addresses := wrapNMDBusIpv6Addresses(addressProp)
-	if len(ipv6Addresses) > 0 {
-		address = ipv6Addresses[0].Address
-		prefix = fmt.Sprintf("%d", ipv6Addresses[0].Prefix)
-	}
-	for _, addr := range ipv6Addresses {
-		gateways = append(gateways, addr.Gateway)
+	addressDataProp, _ := ip6config.AddressData().Get(0)
+	addresses := parseAddressDataProp(addressDataProp)
+
+	ip6Data.Addresses = make([]addressDataItem, 0, len(addresses))
+	for _, addr := range addresses {
 		if addr.Address[:5] != "FE80:" && // link local
 			addr.Address[:5] != "FEC0:" { // site local
-			address = addr.Address
-			prefix = fmt.Sprintf("%d", addr.Prefix)
+			ip6Data.Addresses = append(ip6Data.Addresses, addr)
 		}
 	}
 
-	nameServersProp, _ := ip6config.Nameservers().Get(0)
-	nameServers = wrapIpv6Dns(nameServersProp)
+	if len(ip6Data.Addresses) == 0 {
+		ip6Data.Addresses = []addressDataItem{
+			emptyIPv6,
+		}
+	}
+
+	ip6Data.Gateway, _ = ip6config.Gateway().Get(0)
+
+	nameserversProp, _ := ip6config.Nameservers().Get(0)
+	ip6Data.Nameservers = wrapIpv6Dns(nameserversProp)
 	return
 }
 
@@ -808,20 +856,23 @@ func wrapNMDBusIpv6Addresses(data []nmdbus.IP6Address) (wrapData ipv6AddressesWr
 }
 
 func nmGetDeviceState(devPath dbus.ObjectPath) (state uint32) {
-	dev, err := nmNewDevice(devPath)
+	d, err := nmNewDevice(devPath)
 	if err != nil {
 		return nm.NM_DEVICE_STATE_UNKNOWN
 	}
 
+	dev := d.Device()
 	state, _ = dev.State().Get(0)
 	return
 }
 
 func nmSetDeviceAutoconnect(devPath dbus.ObjectPath, autoConnect bool) {
-	dev, err := nmNewDevice(devPath)
+	d, err := nmNewDevice(devPath)
 	if err != nil {
 		return
 	}
+
+	dev := d.Device()
 	err = dev.Autoconnect().Set(0, autoConnect)
 	if err != nil {
 		logger.Warning("failed to set autoconnect:", err)
@@ -830,10 +881,12 @@ func nmSetDeviceAutoconnect(devPath dbus.ObjectPath, autoConnect bool) {
 }
 
 func nmSetDeviceManaged(devPath dbus.ObjectPath, managed bool) (err error) {
-	dev, err := nmNewDevice(devPath)
+	d, err := nmNewDevice(devPath)
 	if err != nil {
 		return
 	}
+
+	dev := d.Device()
 	err = dev.Managed().Set(0, managed)
 	if err != nil {
 		logger.Warning("failed to set device managed:", err)
@@ -843,20 +896,23 @@ func nmSetDeviceManaged(devPath dbus.ObjectPath, managed bool) (err error) {
 }
 
 func nmGetDeviceType(devPath dbus.ObjectPath) (devType uint32) {
-	dev, err := nmNewDevice(devPath)
+	d, err := nmNewDevice(devPath)
 	if err != nil {
 		return nm.NM_DEVICE_TYPE_UNKNOWN
 	}
 
+	dev := d.Device()
 	devType, _ = dev.DeviceType().Get(0)
 	return
 }
 
 func nmGetDeviceActiveConnection(devPath dbus.ObjectPath) (acPath dbus.ObjectPath) {
-	dev, err := nmNewDevice(devPath)
+	d, err := nmNewDevice(devPath)
 	if err != nil {
 		return
 	}
+
+	dev := d.Device()
 	acPath, _ = dev.ActiveConnection().Get(0)
 	return
 }
