@@ -29,14 +29,16 @@ import (
 	"sync"
 	"time"
 
-	gio "pkg.deepin.io/gir/gio-2.0"
-	"pkg.deepin.io/lib/dbusutil/gsprop"
+	gio "github.com/linuxdeepin/go-gir/gio-2.0"
+	"github.com/linuxdeepin/go-lib/dbusutil/gsprop"
 
+	"github.com/godbus/dbus"
+	"github.com/linuxdeepin/go-lib/dbusutil"
+	"github.com/linuxdeepin/go-lib/gsettings"
+	"github.com/linuxdeepin/go-lib/procfs"
 	x "github.com/linuxdeepin/go-x11-client"
 	xscreensaver "github.com/linuxdeepin/go-x11-client/ext/screensaver"
 	"github.com/linuxdeepin/go-x11-client/util/wm/ewmh"
-	"pkg.deepin.io/lib/gsettings"
-	"pkg.deepin.io/lib/procfs"
 )
 
 const submodulePSP = "PowerSavePlan"
@@ -169,13 +171,21 @@ func (psp *powerSavePlan) Start() error {
 	if err != nil {
 		logger.Warning("failed to connectChanged PowerSavingModeBrightnessDropPercent:", err)
 	}
-	_, err = screenSaver.ConnectIdleOn(psp.HandleIdleOn)
-	if err != nil {
-		logger.Warning("failed to ConnectIdleOn:", err)
-	}
-	_, err = screenSaver.ConnectIdleOff(psp.HandleIdleOff)
-	if err != nil {
-		logger.Warning("failed to ConnectIdleOff:", err)
+	sessionType := os.Getenv("XDG_SESSION_TYPE")
+	if strings.Contains(sessionType, "wayland") {
+		err = psp.ConnectIdle()
+		if err != nil {
+			logger.Warning("failed to ConnectIdleOn:", err)
+		}
+	} else {
+		_, err = screenSaver.ConnectIdleOn(psp.HandleIdleOn)
+		if err != nil {
+			logger.Warning("failed to ConnectIdleOn:", err)
+		}
+		_, err = screenSaver.ConnectIdleOff(psp.HandleIdleOff)
+		if err != nil {
+			logger.Warning("failed to ConnectIdleOff:", err)
+		}
 	}
 	err = display.Brightness().ConnectChanged(psp.handleBrightnessPropertyChanged)
 	if err != nil {
@@ -248,6 +258,9 @@ func (psp *powerSavePlan) handlePowerSavingModeBrightnessDropPercentChanged(hasV
 
 //节能模式变化后的亮度修改
 func (psp *powerSavePlan) handlePowerSavingModeChanged(hasValue bool, enabled bool) {
+	if !psp.manager.isSessionActive() {
+		return
+	}
 	const (
 		multiLevelAdjustmentScale     = 0.2 // 分级调节时，默认按照20%亮度调节值调整亮度，并在亮度显示的设置分级中，归到所在分级
 		multiLevelAdjustmentThreshold = 100 // 分级调节判断阈值，最大亮度值小于该值且不为0时，调节方式为分级调节
@@ -364,9 +377,6 @@ func (mts metaTasks) min() int32 {
 }
 
 func (mts metaTasks) setRealDelay(min int32) {
-	if min == 0 {
-		return
-	}
 	for idx := range mts {
 		t := &mts[idx]
 		nSecs := t.delay - min
@@ -388,8 +398,17 @@ func (psp *powerSavePlan) Update(screenSaverStartDelay, lockDelay,
 		" screenBlackDelay=%vs, sleepDelay=%vs)",
 		screenSaverStartDelay, lockDelay, screenBlackDelay, sleepDelay)
 
-	tasks := make(metaTasks, 0, 4)
-	if screenSaverStartDelay > 0 {
+	// 按照优先级 待机=屏保>关闭显示器=自动锁屏
+	tasks := make(metaTasks, 0, 5)
+
+	if sleepDelay > 0 && canAddToTasks("sleep", sleepDelay, tasks) {
+		tasks = append(tasks, metaTask{
+			name:  "sleep",
+			delay: sleepDelay,
+			fn:    psp.makeSystemSleep,
+		})
+	}
+	if screenSaverStartDelay > 0 && canAddToTasks("screenSaverStart", screenSaverStartDelay, tasks) {
 		tasks = append(tasks, metaTask{
 			name:  "screenSaverStart",
 			delay: screenSaverStartDelay,
@@ -397,7 +416,7 @@ func (psp *powerSavePlan) Update(screenSaverStartDelay, lockDelay,
 		})
 	}
 
-	if lockDelay > 0 {
+	if lockDelay > 0 && canAddToTasks("lock", lockDelay, tasks) {
 		tasks = append(tasks, metaTask{
 			name:  "lock",
 			delay: lockDelay,
@@ -405,7 +424,7 @@ func (psp *powerSavePlan) Update(screenSaverStartDelay, lockDelay,
 		})
 	}
 
-	if screenBlackDelay > 0 {
+	if screenBlackDelay > 0 && canAddToTasks("screenBlack", screenBlackDelay, tasks) {
 		tasks = append(tasks, metaTask{
 			name:  "screenBlack",
 			delay: screenBlackDelay,
@@ -413,14 +432,6 @@ func (psp *powerSavePlan) Update(screenSaverStartDelay, lockDelay,
 		})
 	}
 
-	if sleepDelay > 0 {
-		tasks = append(tasks, metaTask{
-			name:  "sleep",
-			delay: sleepDelay,
-			fn:    psp.makeSystemSleep,
-		})
-	}
-
 	min := tasks.min()
 	tasks.setRealDelay(min)
 	err := psp.setScreenSaverTimeout(min)
@@ -615,22 +626,17 @@ func (psp *powerSavePlan) HandleIdleOn() {
 	logger.Info("HandleIdleOn")
 
 	idleTime := psp.metaTasks.min()
-	xConn, err := x.NewConn()
-	if err == nil {
-		xDefaultScreen := xConn.GetDefaultScreen()
-		if xDefaultScreen != nil {
-			xInfo, err := xscreensaver.QueryInfo(xConn, x.Drawable(xDefaultScreen.Root)).Reply(xConn)
-			if err == nil {
-				idleTime = int32(xInfo.MsSinceUserInput / 1000)
-			} else {
-				logger.Warning(err)
-			}
+	xConn := psp.manager.helper.xConn
+	xDefaultScreen := xConn.GetDefaultScreen()
+	if xDefaultScreen != nil {
+		xInfo, err := xscreensaver.QueryInfo(xConn, x.Drawable(xDefaultScreen.Root)).Reply(xConn)
+		if err == nil {
+			idleTime = int32(xInfo.MsSinceUserInput / 1000)
 		} else {
-			logger.Warning("cannot get X11 default screen")
+			logger.Warning(err)
 		}
-		xConn.Close()
 	} else {
-		logger.Warning(err)
+		logger.Warning("cannot get X11 default screen")
 	}
 
 	logger.Debugf("idle time: %d ms", idleTime)
@@ -668,17 +674,8 @@ func (psp *powerSavePlan) HandleIdleOff() {
 	logger.Info("HandleIdleOff")
 	psp.interruptTasks()
 	psp.manager.setDPMSModeOn()
+	psp.manager.setWmBlackScreenActive(false)
 	psp.resetBrightness()
-
-	_, err := os.Stat("/etc/deepin/no_suspend")
-	if err == nil {
-		if psp.manager.ScreenBlackLock.Get() {
-			//m.setDPMSModeOn()
-			//m.lockWaitShow(4 * time.Second)
-			psp.manager.doLock(false)
-			time.Sleep(time.Millisecond * 500)
-		}
-	}
 }
 
 func (psp *powerSavePlan) isWindowFullScreenAndFocused(xid x.Window) (bool, error) {
@@ -722,6 +719,22 @@ func (psp *powerSavePlan) initMultiBrightnessWithPsm() {
 	}
 
 	psp.setBrightnessFromDisplay()
+	// 开机重启注销等操作后，去掉手动调整亮度的限制条件，关闭节能模式亮度提升
+	brightnessDropPercent, err := psp.manager.helper.Power.PowerSavingModeBrightnessDropPercent().Get(0)
+	if err != nil {
+		logger.Warning("failed to get PowerSavingModeBrightnessDropPercent:", err)
+		return
+	}
+	for _, val := range psp.multiBrightnessWithPsm.MultiBrightness {
+		if val.ManuallyModified == true {
+			val.ManuallyModified = false
+			val.BrightnessSaved = val.BrightnessLatest / (1 - float64(brightnessDropPercent)/100)
+			if val.BrightnessSaved > 1 {
+				val.BrightnessSaved = 1
+			}
+		}
+	}
+	psp.setToBrightnessSave()
 }
 
 func (psp *powerSavePlan) setToBrightnessSave() {
@@ -772,7 +785,7 @@ type brightnessWithPsp struct {
 }
 
 type multiBrightnessWithPsm struct {
-	MultiBrightness []brightnessWithPsp
+	MultiBrightness []*brightnessWithPsp
 	valueTmp        map[string]float64
 }
 
@@ -831,7 +844,7 @@ func (mb *multiBrightnessWithPsm) toObject(b string) error {
 
 func (mb *multiBrightnessWithPsm) mapToObject() {
 	for k, v := range mb.valueTmp {
-		mb.MultiBrightness = append(mb.MultiBrightness, brightnessWithPsp{MonitorName: k, BrightnessSaved: v})
+		mb.MultiBrightness = append(mb.MultiBrightness, &brightnessWithPsp{MonitorName: k, BrightnessSaved: v})
 	}
 }
 
@@ -847,3 +860,82 @@ func (mb *multiBrightnessWithPsm) getReferenceBrightnessWhilePsmPercentChanged(k
 	}
 	return 0, fmt.Errorf("not find Monitor %s's Brightness", key)
 }
+
+// 判断休眠、待机、屏保、锁屏、关闭显示器等任务能否加入任务队列
+// 优先级为：休眠 > 待机=屏保 > 锁屏=关闭显示器
+func canAddToTasks(sType string, delay int32, tasks metaTasks) bool {
+	if len(tasks) == 0 {
+		return true
+	}
+
+	switch sType {
+	case "hibernate":
+		return true
+	case "sleep":
+		if delay < tasks.min() {
+			return true
+		} else {
+			return false
+		}
+	case "screenSaverStart":
+		if delay < tasks.min() {
+			return true
+		} else if delay == tasks.min() && tasks[len(tasks)-1].name == "sleep" {
+			return true
+		} else {
+			return false
+		}
+
+	case "lock":
+		if delay < tasks.min() {
+			return true
+		} else {
+			return false
+		}
+	case "screenBlack":
+		if delay < tasks.min() {
+			return true
+		} else if delay == tasks.min() && tasks[len(tasks)-1].name == "lock" {
+			return true
+		} else {
+			return false
+		}
+	default:
+		return false
+	}
+}
+
+func (psp *powerSavePlan) ConnectIdle() error {
+	sessionBus, err := dbus.SessionBus()
+	if err != nil {
+		logger.Warning(err)
+		return err
+	}
+	err = sessionBus.Object("com.deepin.daemon.KWayland",
+		"/com/deepin/daemon/KWayland/Output").AddMatchSignal("com.deepin.daemon.KWayland.Idle", "IdleTimeout").Err
+	if err != nil {
+		logger.Warning(err)
+		return err
+	}
+	sessionSigLoop := dbusutil.NewSignalLoop(sessionBus, 10)
+	sessionSigLoop.AddHandler(&dbusutil.SignalRule{
+		Name: "com.deepin.daemon.KWayland.Idle.IdleTimeout",
+	}, func(sig *dbus.Signal) {
+		if strings.HasPrefix(string(sig.Path),
+			"/com/deepin/daemon/KWayland/") &&
+			len(sig.Body) == 1 {
+			bIdle, ok := sig.Body[0].(bool)
+			if !ok {
+				return
+			}
+			if bIdle {
+				psp.HandleIdleOn()
+			} else {
+				psp.HandleIdleOff()
+			}
+		}
+	})
+	sessionSigLoop.Start()
+
+	return nil
+}
