@@ -21,16 +21,15 @@ package power
 
 import (
 	"math"
-	"os"
 	"os/exec"
 	"time"
 
 	dbus "github.com/godbus/dbus"
+	"github.com/linuxdeepin/dde-api/soundutils"
+	. "github.com/linuxdeepin/go-lib/gettext"
+	"github.com/linuxdeepin/go-lib/gsettings"
+	"github.com/linuxdeepin/go-lib/pulse"
 	"github.com/linuxdeepin/go-x11-client/ext/dpms"
-	"pkg.deepin.io/dde/api/soundutils"
-	. "pkg.deepin.io/lib/gettext"
-	"pkg.deepin.io/lib/gsettings"
-	"pkg.deepin.io/lib/pulse"
 )
 
 func (m *Manager) waitLockShowing(timeout time.Duration) {
@@ -73,6 +72,35 @@ func (m *Manager) lockWaitShow(timeout time.Duration, autoStartAuth bool) {
 	m.waitLockShowing(timeout)
 }
 
+func (m *Manager) isWmBlackScreenActive() bool {
+	bus, err := dbus.SessionBus()
+	if err == nil {
+		kwinInter := bus.Object("org.kde.KWin", "/BlackScreen")
+		var active bool
+		err = kwinInter.Call("org.kde.kwin.BlackScreen.getActive",
+			dbus.FlagNoAutoStart).Store(&active)
+		if err != nil {
+			logger.Warning("failed to get kwin blackscreen effect active:", err)
+			return false
+		}
+		return active
+	} else {
+		return false
+	}
+}
+
+func (m *Manager) setWmBlackScreenActive(active bool) {
+	logger.Info("set blackScreen effect active: ", active)
+	bus, err := dbus.SessionBus()
+	if err == nil {
+		kwinInter := bus.Object("org.kde.KWin", "/BlackScreen")
+		err = kwinInter.Call("org.kde.kwin.BlackScreen.setActive", 0, active).Err
+		if err != nil {
+			logger.Warning("set blackScreen active failed:", err)
+		}
+	}
+}
+
 func (m *Manager) setDPMSModeOn() {
 	logger.Info("DPMS On")
 	var err error
@@ -123,10 +151,6 @@ func (m *Manager) doLock(autoStartAuth bool) {
 }
 
 func (m *Manager) canSuspend() bool {
-	if os.Getenv("POWER_CAN_SLEEP") == "0" {
-		logger.Info("env POWER_CAN_SLEEP == 0")
-		return false
-	}
 	can, err := m.helper.SessionManager.CanSuspend(0)
 	if err != nil {
 		logger.Warning(err)
@@ -162,6 +186,15 @@ func (m *Manager) doSuspendByFront() {
 	}
 }
 
+func (m *Manager) canShutdown() bool {
+	can, err := m.helper.SessionManager.CanShutdown(0)
+	if err != nil {
+		logger.Warning(err)
+		return false
+	}
+	return can
+}
+
 func (m *Manager) doShutdown() {
 	sessionManager := m.helper.SessionManager
 	can, err := sessionManager.CanShutdown(0)
@@ -183,10 +216,6 @@ func (m *Manager) doShutdown() {
 }
 
 func (m *Manager) canHibernate() bool {
-	if os.Getenv("POWER_CAN_HIBERNATE") == "0" {
-		logger.Info("env POWER_CAN_HIBERNATE == 0")
-		return false
-	}
 	can, err := m.helper.SessionManager.CanHibernate(0)
 	if err != nil {
 		logger.Warning(err)
@@ -200,7 +229,6 @@ func (m *Manager) doHibernate() {
 		logger.Info("can not hibernate")
 		return
 	}
-
 	logger.Debug("hibernate")
 	err := m.helper.SessionManager.RequestHibernate(0)
 	if err != nil {
@@ -223,8 +251,11 @@ func (m *Manager) doHibernateByFront() {
 
 func (m *Manager) doTurnOffScreen() {
 	if m.ScreenBlackLock.Get() {
+		logger.Info("Show lock")
 		m.doLock(true)
+		time.Sleep(1 * time.Second)
 	}
+
 	logger.Info("Turn off screen")
 	m.setDPMSModeOff()
 }
@@ -389,6 +420,15 @@ func (m *Manager) initGSettingsConnectChanged() {
 			value := m.BatteryPressPowerBtnAction.Get()
 			notifyString := getNotifyString(settingKeyBatteryPressPowerBtnAction, value)
 			m.sendChangeNotify(powerSettingsIcon, Tr("Power settings changed"), notifyString)
+		case settingKeyHighPerformanceEnabled:
+			//根据systemPower::IsHighPerformanceEnabled GSetting::settingKeyHighPerformanceEnabled
+			bSettingKeyHighPerformanceEnabled := m.settings.GetBoolean(settingKeyHighPerformanceEnabled)
+			if bSettingKeyHighPerformanceEnabled == m.gsHighPerformanceEnabled {
+				return
+			}
+			m.gsHighPerformanceEnabled = bSettingKeyHighPerformanceEnabled
+			isHighPerformanceSupported, _ := m.systemPower.IsHighPerformanceSupported().Get(0)
+			m.setPropIsHighPerformanceSupported(isHighPerformanceSupported && bSettingKeyHighPerformanceEnabled)
 		}
 	})
 }
