@@ -18,7 +18,12 @@
  */
 
 #include <stdio.h>
+#include <stdint.h>
 #include <math.h>
+#include <errno.h>
+#include <syslog.h>
+#include <libinput.h>
+#include <string.h>
 
 #include <glib.h>
 #include <poll.h>
@@ -28,9 +33,9 @@
 #include "_cgo_export.h"
 
 #define ALARM_TIMEOUT_DEFAULT 700 // 700ms
-#define LONG_PRESS_MAX_DISTANCE 1
-#define SCREEN_WIDTH 100;
-#define SCREEN_HEIGHT 100;
+#define LONG_PRESS_MAX_DISTANCE 3
+#define SCREEN_WIDTH 100
+#define SCREEN_HEIGHT 100
 
 struct raw_multitouch_event {
     double dx_unaccel, dy_unaccel;
@@ -39,6 +44,7 @@ struct raw_multitouch_event {
     uint64_t t_start_tap;
     guint tap_id;
     bool dblclick;
+    bool ignore;
 };
 
 static void raw_event_reset(struct raw_multitouch_event *event, bool reset_dblclick);
@@ -80,6 +86,10 @@ static double long_press_distance = LONG_PRESS_MAX_DISTANCE;
 static int short_press_duration = 200;
 static int dblclick_duration = 0; // 判断两次单击的间隔，是否为双击
 
+static uint64_t _prev_ev_time; // 前一个非 TOUCH_FRAME 事件的时间，单位 usec
+static int _prev_ev_type; // 前一个非 TOUCH_FRAME 事件的类型
+static uint64_t _prev_frame_ev_time; // 前一个 TOUCH_FRAME 事件的时间，单位 usec
+
 int
 start_loop(int verbose, double distance)
 {
@@ -116,7 +126,12 @@ start_loop(int verbose, double distance)
     fds.revents = 0;
 
     quit = 0;
-    while(!quit && poll(&fds, 1, -1) > -1) {
+    while(!quit) {
+        if(poll(&fds, 1, -1) < 0)
+        {
+            continue;
+        }
+
         handle_events(li, movements);
         handle_movements(movements);        //handle touch screen
     }
@@ -471,11 +486,43 @@ handle_touch_events(struct libinput_event *ev, int ty,struct movement *m)
 {
     point scale;
     struct libinput_device *dev = libinput_event_get_device(ev);
+    const char* node= NULL;
+    struct raw_multitouch_event *rme = NULL;
+
     if (!dev) {
         fprintf(stderr, "Get device from event failure\n");
         return ;
     }
 
+    node = get_multitouch_device_node(ev);
+    rme = g_hash_table_lookup(ev_table, node);
+    if (rme && rme->ignore) {
+        return;
+    }
+
+    struct libinput_event_touch *tevent = libinput_event_get_touch_event(ev);
+    uint64_t time_usec = libinput_event_touch_get_time_usec(tevent);
+    g_debug("touch event %d time usec: %llu", ty, time_usec);
+
+    // 为修复 pms bug 64939，发现从 libinput 获取到的事件被重复了一遍，比如一次按下，
+    // 会出现 TOUCH_DOWN,TOUCH_FRAME,TOUCH_DOWN,TOUCH_FRAME 这样的，于是写了规避的代码。
+    // 具体原因可能是 libinput 的 bug，有待调查。
+    if (_prev_ev_type == ty && _prev_ev_time == time_usec && _prev_frame_ev_time == time_usec) {
+        // 主要特征是时间是相同的
+        g_debug("ignore repeat event");
+        _prev_ev_type = 0;
+        _prev_ev_time = 0;
+        _prev_frame_ev_time = 0;
+        return;
+    }
+
+    if (ty == LIBINPUT_EVENT_TOUCH_FRAME) {
+       _prev_frame_ev_time = time_usec;
+    } else {
+        _prev_ev_time = time_usec;
+        _prev_ev_type = ty;
+    }
+
     switch (ty) {
     case LIBINPUT_EVENT_TOUCH_MOTION:{
         handle_touch_event_motion(ev, m);
@@ -559,6 +606,38 @@ handle_touch_events(struct libinput_event *ev, int ty,struct movement *m)
     }
 }
 
+static void
+handle_mouse_events(struct libinput_event *ev, int type)
+{
+    struct libinput_device *dev = libinput_event_get_device(ev);
+    if (!dev) {
+        fprintf(stderr, "Get device from event failure\n");
+        return ;
+    }
+
+    struct libinput_event_pointer *mouse = libinput_event_get_pointer_event(ev);
+    enum libinput_pointer_axis_source source = libinput_event_pointer_get_axis_source(mouse);
+    double value = libinput_event_pointer_get_axis_value(mouse, LIBINPUT_POINTER_AXIS_SCROLL_HORIZONTAL);
+
+    handleMouseEvent(type, source,value);
+}
+
+static void
+handle_keyboard_events(struct libinput_event *ev, int type)
+{
+    struct libinput_device *dev = libinput_event_get_device(ev);
+    if (!dev) {
+        fprintf(stderr, "Get device from event failure\n");
+        return ;
+    }
+
+    struct libinput_event_keyboard *keyboard = libinput_event_get_keyboard_event(ev);
+    uint32_t key = libinput_event_keyboard_get_key(keyboard);
+    enum libinput_key_state state = libinput_event_keyboard_get_key_state(keyboard);
+
+    handleKeyboardEvent(key,(uint32_t)state);
+}
+
 static void
 handle_events(struct libinput *li, struct movement *m)
 {
@@ -601,7 +680,22 @@ handle_events(struct libinput *li, struct movement *m)
         case LIBINPUT_EVENT_TOUCH_DOWN:
         case LIBINPUT_EVENT_TOUCH_CANCEL:
         case LIBINPUT_EVENT_TOUCH_FRAME:{
-            handle_touch_events(ev, type, m);  
+            handle_touch_events(ev, type, m);
+            break;
+        }
+        case LIBINPUT_EVENT_KEYBOARD_KEY: {
+            if (NULL != getenv("WAYLAND_DISPLAY")) {
+                handle_keyboard_events(ev, type);
+            }
+            break;
+        }
+        case LIBINPUT_EVENT_POINTER_MOTION:
+        case LIBINPUT_EVENT_POINTER_MOTION_ABSOLUTE:
+        case LIBINPUT_EVENT_POINTER_BUTTON:
+        case LIBINPUT_EVENT_POINTER_AXIS: {
+            if (NULL != getenv("WAYLAND_DISPLAY")) {
+                handle_mouse_events(ev, type);
+            }
             break;
         }
         default:
@@ -611,3 +705,11 @@ handle_events(struct libinput *li, struct movement *m)
         libinput_dispatch(li);
     }
 }
+
+void set_device_ignore(const char* node, bool ignore) {
+    struct raw_multitouch_event *rme = g_hash_table_lookup(ev_table, node);
+    if (rme) {
+        g_debug("[gesture] set device ignore: %s", ignore ? "true" : "false");
+        rme->ignore = ignore;
+    }
+}
